# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

METRIC_EXTREMA_METADATA = Metadata(
    id="a62651df948001aa8493b5d383ecb5de960f26cc.boutiques",
    name="metric-extrema",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


MetricExtremaPresmoothParameters = typing.TypedDict('MetricExtremaPresmoothParameters', {
    "@type": typing.NotRequired[typing.Literal["presmooth"]],
    "kernel": float,
    "opt_fwhm": bool,
})
MetricExtremaPresmoothParametersTagged = typing.TypedDict('MetricExtremaPresmoothParametersTagged', {
    "@type": typing.Literal["presmooth"],
    "kernel": float,
    "opt_fwhm": bool,
})


MetricExtremaThresholdParameters = typing.TypedDict('MetricExtremaThresholdParameters', {
    "@type": typing.NotRequired[typing.Literal["threshold"]],
    "low": float,
    "high": float,
})
MetricExtremaThresholdParametersTagged = typing.TypedDict('MetricExtremaThresholdParametersTagged', {
    "@type": typing.Literal["threshold"],
    "low": float,
    "high": float,
})


MetricExtremaParameters = typing.TypedDict('MetricExtremaParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/metric-extrema"]],
    "surface": InputPathType,
    "metric_in": InputPathType,
    "distance": float,
    "metric_out": str,
    "presmooth": typing.NotRequired[MetricExtremaPresmoothParameters | None],
    "opt_roi_roi_metric": typing.NotRequired[InputPathType | None],
    "threshold": typing.NotRequired[MetricExtremaThresholdParameters | None],
    "opt_sum_columns": bool,
    "opt_consolidate_mode": bool,
    "opt_only_maxima": bool,
    "opt_only_minima": bool,
    "opt_column_column": typing.NotRequired[str | None],
})
MetricExtremaParametersTagged = typing.TypedDict('MetricExtremaParametersTagged', {
    "@type": typing.Literal["workbench/metric-extrema"],
    "surface": InputPathType,
    "metric_in": InputPathType,
    "distance": float,
    "metric_out": str,
    "presmooth": typing.NotRequired[MetricExtremaPresmoothParameters | None],
    "opt_roi_roi_metric": typing.NotRequired[InputPathType | None],
    "threshold": typing.NotRequired[MetricExtremaThresholdParameters | None],
    "opt_sum_columns": bool,
    "opt_consolidate_mode": bool,
    "opt_only_maxima": bool,
    "opt_only_minima": bool,
    "opt_column_column": typing.NotRequired[str | None],
})


def metric_extrema_presmooth_params(
    kernel: float,
    opt_fwhm: bool = False,
) -> MetricExtremaPresmoothParametersTagged:
    """
    Build parameters.
    
    Args:
        kernel: the size of the gaussian smoothing kernel in mm, as sigma by\
            default.
        opt_fwhm: kernel size is FWHM, not sigma.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "presmooth",
        "kernel": kernel,
        "opt_fwhm": opt_fwhm,
    }
    return params


def metric_extrema_presmooth_cargs(
    params: MetricExtremaPresmoothParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-presmooth")
    cargs.append(str(params.get("kernel", None)))
    if params.get("opt_fwhm", False):
        cargs.append("-fwhm")
    return cargs


def metric_extrema_threshold_params(
    low: float,
    high: float,
) -> MetricExtremaThresholdParametersTagged:
    """
    Build parameters.
    
    Args:
        low: the largest value to consider for being a minimum.
        high: the smallest value to consider for being a maximum.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "threshold",
        "low": low,
        "high": high,
    }
    return params


def metric_extrema_threshold_cargs(
    params: MetricExtremaThresholdParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-threshold")
    cargs.append(str(params.get("low", None)))
    cargs.append(str(params.get("high", None)))
    return cargs


class MetricExtremaOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetricExtremaParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output extrema metric"""


def metric_extrema_params(
    surface: InputPathType,
    metric_in: InputPathType,
    distance: float,
    metric_out: str,
    presmooth: MetricExtremaPresmoothParameters | None = None,
    opt_roi_roi_metric: InputPathType | None = None,
    threshold: MetricExtremaThresholdParameters | None = None,
    opt_sum_columns: bool = False,
    opt_consolidate_mode: bool = False,
    opt_only_maxima: bool = False,
    opt_only_minima: bool = False,
    opt_column_column: str | None = None,
) -> MetricExtremaParametersTagged:
    """
    Build parameters.
    
    Args:
        surface: the surface to use for distance information.
        metric_in: the metric to find the extrema of.
        distance: the minimum distance between identified extrema of the same\
            type.
        metric_out: the output extrema metric.
        presmooth: smooth the metric before finding extrema.
        opt_roi_roi_metric: ignore values outside the selected area: the area\
            to find extrema in, as a metric.
        threshold: ignore small extrema.
        opt_sum_columns: output the sum of the extrema columns instead of each\
            column separately.
        opt_consolidate_mode: use consolidation of local minima instead of a\
            large neighborhood.
        opt_only_maxima: only find the maxima.
        opt_only_minima: only find the minima.
        opt_column_column: select a single column to find extrema in: the\
            column number or name.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/metric-extrema",
        "surface": surface,
        "metric_in": metric_in,
        "distance": distance,
        "metric_out": metric_out,
        "opt_sum_columns": opt_sum_columns,
        "opt_consolidate_mode": opt_consolidate_mode,
        "opt_only_maxima": opt_only_maxima,
        "opt_only_minima": opt_only_minima,
    }
    if presmooth is not None:
        params["presmooth"] = presmooth
    if opt_roi_roi_metric is not None:
        params["opt_roi_roi_metric"] = opt_roi_roi_metric
    if threshold is not None:
        params["threshold"] = threshold
    if opt_column_column is not None:
        params["opt_column_column"] = opt_column_column
    return params


def metric_extrema_cargs(
    params: MetricExtremaParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-metric-extrema")
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(execution.input_file(params.get("metric_in", None)))
    cargs.append(str(params.get("distance", None)))
    cargs.append(params.get("metric_out", None))
    if params.get("presmooth", None) is not None:
        cargs.extend(metric_extrema_presmooth_cargs(params.get("presmooth", None), execution))
    if params.get("opt_roi_roi_metric", None) is not None:
        cargs.extend([
            "-roi",
            execution.input_file(params.get("opt_roi_roi_metric", None))
        ])
    if params.get("threshold", None) is not None:
        cargs.extend(metric_extrema_threshold_cargs(params.get("threshold", None), execution))
    if params.get("opt_sum_columns", False):
        cargs.append("-sum-columns")
    if params.get("opt_consolidate_mode", False):
        cargs.append("-consolidate-mode")
    if params.get("opt_only_maxima", False):
        cargs.append("-only-maxima")
    if params.get("opt_only_minima", False):
        cargs.append("-only-minima")
    if params.get("opt_column_column", None) is not None:
        cargs.extend([
            "-column",
            params.get("opt_column_column", None)
        ])
    return cargs


def metric_extrema_outputs(
    params: MetricExtremaParameters,
    execution: Execution,
) -> MetricExtremaOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetricExtremaOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric_out", None)),
    )
    return ret


def metric_extrema_execute(
    params: MetricExtremaParameters,
    runner: Runner | None = None,
) -> MetricExtremaOutputs:
    """
    metric-extrema
    
    Find extrema in a metric file.
    
    Finds extrema in a metric file, such that no two extrema of the same type
    are within <distance> of each other. The extrema are labeled as -1 for
    minima, 1 for maxima, 0 otherwise. If -only-maxima or -only-minima is
    specified, then it will ignore extrema not of the specified type. These
    options are mutually exclusive.
    
    If -roi is specified, not only is data outside the roi not used, but any
    vertex on the edge of the ROI will never be counted as an extrema, in case
    the ROI cuts across a gradient, which would otherwise generate extrema where
    there should be none.
    
    If -sum-columns is specified, these extrema columns are summed, and the
    output has a single column with this result.
    
    By default, a datapoint is an extrema only if it is more extreme than every
    other datapoint that is within <distance> from it. If -consolidate-mode is
    used, it instead starts by finding all datapoints that are more extreme than
    their immediate neighbors, then while there are any extrema within
    <distance> of each other, take the two extrema closest to each other and
    merge them into one by a weighted average based on how many original extrema
    have been merged into each.
    
    By default, all input columns are used with no smoothing, use -column to
    specify a single column to use, and -presmooth to smooth the input before
    finding the extrema.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricExtremaOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_EXTREMA_METADATA)
    params = execution.params(params)
    cargs = metric_extrema_cargs(params, execution)
    ret = metric_extrema_outputs(params, execution)
    execution.run(cargs)
    return ret


def metric_extrema(
    surface: InputPathType,
    metric_in: InputPathType,
    distance: float,
    metric_out: str,
    presmooth: MetricExtremaPresmoothParameters | None = None,
    opt_roi_roi_metric: InputPathType | None = None,
    threshold: MetricExtremaThresholdParameters | None = None,
    opt_sum_columns: bool = False,
    opt_consolidate_mode: bool = False,
    opt_only_maxima: bool = False,
    opt_only_minima: bool = False,
    opt_column_column: str | None = None,
    runner: Runner | None = None,
) -> MetricExtremaOutputs:
    """
    metric-extrema
    
    Find extrema in a metric file.
    
    Finds extrema in a metric file, such that no two extrema of the same type
    are within <distance> of each other. The extrema are labeled as -1 for
    minima, 1 for maxima, 0 otherwise. If -only-maxima or -only-minima is
    specified, then it will ignore extrema not of the specified type. These
    options are mutually exclusive.
    
    If -roi is specified, not only is data outside the roi not used, but any
    vertex on the edge of the ROI will never be counted as an extrema, in case
    the ROI cuts across a gradient, which would otherwise generate extrema where
    there should be none.
    
    If -sum-columns is specified, these extrema columns are summed, and the
    output has a single column with this result.
    
    By default, a datapoint is an extrema only if it is more extreme than every
    other datapoint that is within <distance> from it. If -consolidate-mode is
    used, it instead starts by finding all datapoints that are more extreme than
    their immediate neighbors, then while there are any extrema within
    <distance> of each other, take the two extrema closest to each other and
    merge them into one by a weighted average based on how many original extrema
    have been merged into each.
    
    By default, all input columns are used with no smoothing, use -column to
    specify a single column to use, and -presmooth to smooth the input before
    finding the extrema.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        surface: the surface to use for distance information.
        metric_in: the metric to find the extrema of.
        distance: the minimum distance between identified extrema of the same\
            type.
        metric_out: the output extrema metric.
        presmooth: smooth the metric before finding extrema.
        opt_roi_roi_metric: ignore values outside the selected area: the area\
            to find extrema in, as a metric.
        threshold: ignore small extrema.
        opt_sum_columns: output the sum of the extrema columns instead of each\
            column separately.
        opt_consolidate_mode: use consolidation of local minima instead of a\
            large neighborhood.
        opt_only_maxima: only find the maxima.
        opt_only_minima: only find the minima.
        opt_column_column: select a single column to find extrema in: the\
            column number or name.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricExtremaOutputs`).
    """
    params = metric_extrema_params(
        surface=surface,
        metric_in=metric_in,
        distance=distance,
        metric_out=metric_out,
        presmooth=presmooth,
        opt_roi_roi_metric=opt_roi_roi_metric,
        threshold=threshold,
        opt_sum_columns=opt_sum_columns,
        opt_consolidate_mode=opt_consolidate_mode,
        opt_only_maxima=opt_only_maxima,
        opt_only_minima=opt_only_minima,
        opt_column_column=opt_column_column,
    )
    return metric_extrema_execute(params, runner)


__all__ = [
    "METRIC_EXTREMA_METADATA",
    "MetricExtremaOutputs",
    "metric_extrema",
    "metric_extrema_execute",
    "metric_extrema_params",
    "metric_extrema_presmooth_params",
    "metric_extrema_threshold_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_CREATE_DENSE_TIMESERIES_METADATA = Metadata(
    id="0c5c29de3b046d189f8d02590ac94b688fe175f9.boutiques",
    name="cifti-create-dense-timeseries",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


CiftiCreateDenseTimeseriesVolumeParameters = typing.TypedDict('CiftiCreateDenseTimeseriesVolumeParameters', {
    "@type": typing.NotRequired[typing.Literal["volume"]],
    "volume_data": InputPathType,
    "structure_label_volume": InputPathType,
})
CiftiCreateDenseTimeseriesVolumeParametersTagged = typing.TypedDict('CiftiCreateDenseTimeseriesVolumeParametersTagged', {
    "@type": typing.Literal["volume"],
    "volume_data": InputPathType,
    "structure_label_volume": InputPathType,
})


CiftiCreateDenseTimeseriesLeftMetricParameters = typing.TypedDict('CiftiCreateDenseTimeseriesLeftMetricParameters', {
    "@type": typing.NotRequired[typing.Literal["left_metric"]],
    "metric": InputPathType,
    "opt_roi_left_roi_metric": typing.NotRequired[InputPathType | None],
})
CiftiCreateDenseTimeseriesLeftMetricParametersTagged = typing.TypedDict('CiftiCreateDenseTimeseriesLeftMetricParametersTagged', {
    "@type": typing.Literal["left_metric"],
    "metric": InputPathType,
    "opt_roi_left_roi_metric": typing.NotRequired[InputPathType | None],
})


CiftiCreateDenseTimeseriesRightMetricParameters = typing.TypedDict('CiftiCreateDenseTimeseriesRightMetricParameters', {
    "@type": typing.NotRequired[typing.Literal["right_metric"]],
    "metric": InputPathType,
    "opt_roi_right_roi_metric": typing.NotRequired[InputPathType | None],
})
CiftiCreateDenseTimeseriesRightMetricParametersTagged = typing.TypedDict('CiftiCreateDenseTimeseriesRightMetricParametersTagged', {
    "@type": typing.Literal["right_metric"],
    "metric": InputPathType,
    "opt_roi_right_roi_metric": typing.NotRequired[InputPathType | None],
})


CiftiCreateDenseTimeseriesCerebellumMetricParameters = typing.TypedDict('CiftiCreateDenseTimeseriesCerebellumMetricParameters', {
    "@type": typing.NotRequired[typing.Literal["cerebellum_metric"]],
    "metric": InputPathType,
    "opt_roi_cerebellum_roi_metric": typing.NotRequired[InputPathType | None],
})
CiftiCreateDenseTimeseriesCerebellumMetricParametersTagged = typing.TypedDict('CiftiCreateDenseTimeseriesCerebellumMetricParametersTagged', {
    "@type": typing.Literal["cerebellum_metric"],
    "metric": InputPathType,
    "opt_roi_cerebellum_roi_metric": typing.NotRequired[InputPathType | None],
})


CiftiCreateDenseTimeseriesParameters = typing.TypedDict('CiftiCreateDenseTimeseriesParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/cifti-create-dense-timeseries"]],
    "cifti_out": str,
    "volume": typing.NotRequired[CiftiCreateDenseTimeseriesVolumeParameters | None],
    "left_metric": typing.NotRequired[CiftiCreateDenseTimeseriesLeftMetricParameters | None],
    "right_metric": typing.NotRequired[CiftiCreateDenseTimeseriesRightMetricParameters | None],
    "cerebellum_metric": typing.NotRequired[CiftiCreateDenseTimeseriesCerebellumMetricParameters | None],
    "opt_timestep_interval": typing.NotRequired[float | None],
    "opt_timestart_start": typing.NotRequired[float | None],
    "opt_unit_unit": typing.NotRequired[str | None],
})
CiftiCreateDenseTimeseriesParametersTagged = typing.TypedDict('CiftiCreateDenseTimeseriesParametersTagged', {
    "@type": typing.Literal["workbench/cifti-create-dense-timeseries"],
    "cifti_out": str,
    "volume": typing.NotRequired[CiftiCreateDenseTimeseriesVolumeParameters | None],
    "left_metric": typing.NotRequired[CiftiCreateDenseTimeseriesLeftMetricParameters | None],
    "right_metric": typing.NotRequired[CiftiCreateDenseTimeseriesRightMetricParameters | None],
    "cerebellum_metric": typing.NotRequired[CiftiCreateDenseTimeseriesCerebellumMetricParameters | None],
    "opt_timestep_interval": typing.NotRequired[float | None],
    "opt_timestart_start": typing.NotRequired[float | None],
    "opt_unit_unit": typing.NotRequired[str | None],
})


def cifti_create_dense_timeseries_volume_params(
    volume_data: InputPathType,
    structure_label_volume: InputPathType,
) -> CiftiCreateDenseTimeseriesVolumeParametersTagged:
    """
    Build parameters.
    
    Args:
        volume_data: volume file containing all voxel data for all volume\
            structures.
        structure_label_volume: label volume file containing labels for cifti\
            structures.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "volume",
        "volume_data": volume_data,
        "structure_label_volume": structure_label_volume,
    }
    return params


def cifti_create_dense_timeseries_volume_cargs(
    params: CiftiCreateDenseTimeseriesVolumeParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-volume")
    cargs.append(execution.input_file(params.get("volume_data", None)))
    cargs.append(execution.input_file(params.get("structure_label_volume", None)))
    return cargs


def cifti_create_dense_timeseries_left_metric_params(
    metric: InputPathType,
    opt_roi_left_roi_metric: InputPathType | None = None,
) -> CiftiCreateDenseTimeseriesLeftMetricParametersTagged:
    """
    Build parameters.
    
    Args:
        metric: the metric file.
        opt_roi_left_roi_metric: roi of vertices to use from left surface: the\
            ROI as a metric file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "left_metric",
        "metric": metric,
    }
    if opt_roi_left_roi_metric is not None:
        params["opt_roi_left_roi_metric"] = opt_roi_left_roi_metric
    return params


def cifti_create_dense_timeseries_left_metric_cargs(
    params: CiftiCreateDenseTimeseriesLeftMetricParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-left-metric")
    cargs.append(execution.input_file(params.get("metric", None)))
    if params.get("opt_roi_left_roi_metric", None) is not None:
        cargs.extend([
            "-roi-left",
            execution.input_file(params.get("opt_roi_left_roi_metric", None))
        ])
    return cargs


def cifti_create_dense_timeseries_right_metric_params(
    metric: InputPathType,
    opt_roi_right_roi_metric: InputPathType | None = None,
) -> CiftiCreateDenseTimeseriesRightMetricParametersTagged:
    """
    Build parameters.
    
    Args:
        metric: the metric file.
        opt_roi_right_roi_metric: roi of vertices to use from right surface:\
            the ROI as a metric file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "right_metric",
        "metric": metric,
    }
    if opt_roi_right_roi_metric is not None:
        params["opt_roi_right_roi_metric"] = opt_roi_right_roi_metric
    return params


def cifti_create_dense_timeseries_right_metric_cargs(
    params: CiftiCreateDenseTimeseriesRightMetricParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-right-metric")
    cargs.append(execution.input_file(params.get("metric", None)))
    if params.get("opt_roi_right_roi_metric", None) is not None:
        cargs.extend([
            "-roi-right",
            execution.input_file(params.get("opt_roi_right_roi_metric", None))
        ])
    return cargs


def cifti_create_dense_timeseries_cerebellum_metric_params(
    metric: InputPathType,
    opt_roi_cerebellum_roi_metric: InputPathType | None = None,
) -> CiftiCreateDenseTimeseriesCerebellumMetricParametersTagged:
    """
    Build parameters.
    
    Args:
        metric: the metric file.
        opt_roi_cerebellum_roi_metric: roi of vertices to use from right\
            surface: the ROI as a metric file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cerebellum_metric",
        "metric": metric,
    }
    if opt_roi_cerebellum_roi_metric is not None:
        params["opt_roi_cerebellum_roi_metric"] = opt_roi_cerebellum_roi_metric
    return params


def cifti_create_dense_timeseries_cerebellum_metric_cargs(
    params: CiftiCreateDenseTimeseriesCerebellumMetricParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-cerebellum-metric")
    cargs.append(execution.input_file(params.get("metric", None)))
    if params.get("opt_roi_cerebellum_roi_metric", None) is not None:
        cargs.extend([
            "-roi-cerebellum",
            execution.input_file(params.get("opt_roi_cerebellum_roi_metric", None))
        ])
    return cargs


class CiftiCreateDenseTimeseriesOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiCreateDenseTimeseriesParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti file"""


def cifti_create_dense_timeseries_params(
    cifti_out: str,
    volume: CiftiCreateDenseTimeseriesVolumeParameters | None = None,
    left_metric: CiftiCreateDenseTimeseriesLeftMetricParameters | None = None,
    right_metric: CiftiCreateDenseTimeseriesRightMetricParameters | None = None,
    cerebellum_metric: CiftiCreateDenseTimeseriesCerebellumMetricParameters | None = None,
    opt_timestep_interval: float | None = None,
    opt_timestart_start: float | None = None,
    opt_unit_unit: str | None = None,
) -> CiftiCreateDenseTimeseriesParametersTagged:
    """
    Build parameters.
    
    Args:
        cifti_out: the output cifti file.
        volume: volume component.
        left_metric: metric for left surface.
        right_metric: metric for left surface.
        cerebellum_metric: metric for the cerebellum.
        opt_timestep_interval: set the timestep: the timestep, in seconds\
            (default 1.0).
        opt_timestart_start: set the start time: the time at the first frame,\
            in seconds (default 0.0).
        opt_unit_unit: use a unit other than time: unit identifier (default\
            SECOND).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-create-dense-timeseries",
        "cifti_out": cifti_out,
    }
    if volume is not None:
        params["volume"] = volume
    if left_metric is not None:
        params["left_metric"] = left_metric
    if right_metric is not None:
        params["right_metric"] = right_metric
    if cerebellum_metric is not None:
        params["cerebellum_metric"] = cerebellum_metric
    if opt_timestep_interval is not None:
        params["opt_timestep_interval"] = opt_timestep_interval
    if opt_timestart_start is not None:
        params["opt_timestart_start"] = opt_timestart_start
    if opt_unit_unit is not None:
        params["opt_unit_unit"] = opt_unit_unit
    return params


def cifti_create_dense_timeseries_cargs(
    params: CiftiCreateDenseTimeseriesParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-cifti-create-dense-timeseries")
    cargs.append(params.get("cifti_out", None))
    if params.get("volume", None) is not None:
        cargs.extend(cifti_create_dense_timeseries_volume_cargs(params.get("volume", None), execution))
    if params.get("left_metric", None) is not None:
        cargs.extend(cifti_create_dense_timeseries_left_metric_cargs(params.get("left_metric", None), execution))
    if params.get("right_metric", None) is not None:
        cargs.extend(cifti_create_dense_timeseries_right_metric_cargs(params.get("right_metric", None), execution))
    if params.get("cerebellum_metric", None) is not None:
        cargs.extend(cifti_create_dense_timeseries_cerebellum_metric_cargs(params.get("cerebellum_metric", None), execution))
    if params.get("opt_timestep_interval", None) is not None:
        cargs.extend([
            "-timestep",
            str(params.get("opt_timestep_interval", None))
        ])
    if params.get("opt_timestart_start", None) is not None:
        cargs.extend([
            "-timestart",
            str(params.get("opt_timestart_start", None))
        ])
    if params.get("opt_unit_unit", None) is not None:
        cargs.extend([
            "-unit",
            params.get("opt_unit_unit", None)
        ])
    return cargs


def cifti_create_dense_timeseries_outputs(
    params: CiftiCreateDenseTimeseriesParameters,
    execution: Execution,
) -> CiftiCreateDenseTimeseriesOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiCreateDenseTimeseriesOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti_out", None)),
    )
    return ret


def cifti_create_dense_timeseries_execute(
    params: CiftiCreateDenseTimeseriesParameters,
    runner: Runner | None = None,
) -> CiftiCreateDenseTimeseriesOutputs:
    """
    cifti-create-dense-timeseries
    
    Create a cifti dense timeseries.
    
    All input files must have the same number of columns/subvolumes. Only the
    specified components will be in the output cifti. At least one component
    must be specified.
    
    See -volume-label-import and -volume-help for format details of label volume
    files. The structure-label-volume should have some of the label names from
    this list, all other label names will be ignored:
    
    CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT
    
    The -unit option accepts these values:
    
    SECOND
    HERTZ
    METER
    RADIAN.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiCreateDenseTimeseriesOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_CREATE_DENSE_TIMESERIES_METADATA)
    params = execution.params(params)
    cargs = cifti_create_dense_timeseries_cargs(params, execution)
    ret = cifti_create_dense_timeseries_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_create_dense_timeseries(
    cifti_out: str,
    volume: CiftiCreateDenseTimeseriesVolumeParameters | None = None,
    left_metric: CiftiCreateDenseTimeseriesLeftMetricParameters | None = None,
    right_metric: CiftiCreateDenseTimeseriesRightMetricParameters | None = None,
    cerebellum_metric: CiftiCreateDenseTimeseriesCerebellumMetricParameters | None = None,
    opt_timestep_interval: float | None = None,
    opt_timestart_start: float | None = None,
    opt_unit_unit: str | None = None,
    runner: Runner | None = None,
) -> CiftiCreateDenseTimeseriesOutputs:
    """
    cifti-create-dense-timeseries
    
    Create a cifti dense timeseries.
    
    All input files must have the same number of columns/subvolumes. Only the
    specified components will be in the output cifti. At least one component
    must be specified.
    
    See -volume-label-import and -volume-help for format details of label volume
    files. The structure-label-volume should have some of the label names from
    this list, all other label names will be ignored:
    
    CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT
    
    The -unit option accepts these values:
    
    SECOND
    HERTZ
    METER
    RADIAN.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        cifti_out: the output cifti file.
        volume: volume component.
        left_metric: metric for left surface.
        right_metric: metric for left surface.
        cerebellum_metric: metric for the cerebellum.
        opt_timestep_interval: set the timestep: the timestep, in seconds\
            (default 1.0).
        opt_timestart_start: set the start time: the time at the first frame,\
            in seconds (default 0.0).
        opt_unit_unit: use a unit other than time: unit identifier (default\
            SECOND).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiCreateDenseTimeseriesOutputs`).
    """
    params = cifti_create_dense_timeseries_params(
        cifti_out=cifti_out,
        volume=volume,
        left_metric=left_metric,
        right_metric=right_metric,
        cerebellum_metric=cerebellum_metric,
        opt_timestep_interval=opt_timestep_interval,
        opt_timestart_start=opt_timestart_start,
        opt_unit_unit=opt_unit_unit,
    )
    return cifti_create_dense_timeseries_execute(params, runner)


__all__ = [
    "CIFTI_CREATE_DENSE_TIMESERIES_METADATA",
    "CiftiCreateDenseTimeseriesOutputs",
    "cifti_create_dense_timeseries",
    "cifti_create_dense_timeseries_cerebellum_metric_params",
    "cifti_create_dense_timeseries_execute",
    "cifti_create_dense_timeseries_left_metric_params",
    "cifti_create_dense_timeseries_params",
    "cifti_create_dense_timeseries_right_metric_params",
    "cifti_create_dense_timeseries_volume_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CREATE_SIGNED_DISTANCE_VOLUME_METADATA = Metadata(
    id="a3c155c43199f00044e684ccce6c00d843dcec5a.boutiques",
    name="create-signed-distance-volume",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


CreateSignedDistanceVolumeParameters = typing.TypedDict('CreateSignedDistanceVolumeParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/create-signed-distance-volume"]],
    "surface": InputPathType,
    "refspace": str,
    "outvol": str,
    "opt_roi_out_roi_vol": typing.NotRequired[str | None],
    "opt_fill_value_value": typing.NotRequired[float | None],
    "opt_exact_limit_dist": typing.NotRequired[float | None],
    "opt_approx_limit_dist": typing.NotRequired[float | None],
    "opt_approx_neighborhood_num": typing.NotRequired[int | None],
    "opt_winding_method": typing.NotRequired[str | None],
})
CreateSignedDistanceVolumeParametersTagged = typing.TypedDict('CreateSignedDistanceVolumeParametersTagged', {
    "@type": typing.Literal["workbench/create-signed-distance-volume"],
    "surface": InputPathType,
    "refspace": str,
    "outvol": str,
    "opt_roi_out_roi_vol": typing.NotRequired[str | None],
    "opt_fill_value_value": typing.NotRequired[float | None],
    "opt_exact_limit_dist": typing.NotRequired[float | None],
    "opt_approx_limit_dist": typing.NotRequired[float | None],
    "opt_approx_neighborhood_num": typing.NotRequired[int | None],
    "opt_winding_method": typing.NotRequired[str | None],
})


class CreateSignedDistanceVolumeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CreateSignedDistanceVolumeParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    outvol: OutputPathType
    """the output volume"""
    opt_roi_out_roi_vol: OutputPathType | None
    """output an roi volume of where the output has a computed value: the output
    roi volume"""


def create_signed_distance_volume_params(
    surface: InputPathType,
    refspace: str,
    outvol: str,
    opt_roi_out_roi_vol: str | None = None,
    opt_fill_value_value: float | None = None,
    opt_exact_limit_dist: float | None = None,
    opt_approx_limit_dist: float | None = None,
    opt_approx_neighborhood_num: int | None = None,
    opt_winding_method: str | None = None,
) -> CreateSignedDistanceVolumeParametersTagged:
    """
    Build parameters.
    
    Args:
        surface: the input surface.
        refspace: a volume in the desired output space (dims, spacing, origin).
        outvol: the output volume.
        opt_roi_out_roi_vol: output an roi volume of where the output has a\
            computed value: the output roi volume.
        opt_fill_value_value: specify a value to put in all voxels that don't\
            get assigned a distance: value to fill with (default 0).
        opt_exact_limit_dist: specify distance for exact output: distance in mm\
            (default 5).
        opt_approx_limit_dist: specify distance for approximate output:\
            distance in mm (default 20).
        opt_approx_neighborhood_num: voxel neighborhood for approximate\
            calculation: size of neighborhood cube measured from center to face, in\
            voxels (default 2 = 5x5x5).
        opt_winding_method: winding method for point inside surface test: name\
            of the method (default EVEN_ODD).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/create-signed-distance-volume",
        "surface": surface,
        "refspace": refspace,
        "outvol": outvol,
    }
    if opt_roi_out_roi_vol is not None:
        params["opt_roi_out_roi_vol"] = opt_roi_out_roi_vol
    if opt_fill_value_value is not None:
        params["opt_fill_value_value"] = opt_fill_value_value
    if opt_exact_limit_dist is not None:
        params["opt_exact_limit_dist"] = opt_exact_limit_dist
    if opt_approx_limit_dist is not None:
        params["opt_approx_limit_dist"] = opt_approx_limit_dist
    if opt_approx_neighborhood_num is not None:
        params["opt_approx_neighborhood_num"] = opt_approx_neighborhood_num
    if opt_winding_method is not None:
        params["opt_winding_method"] = opt_winding_method
    return params


def create_signed_distance_volume_cargs(
    params: CreateSignedDistanceVolumeParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-create-signed-distance-volume")
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(params.get("refspace", None))
    cargs.append(params.get("outvol", None))
    if params.get("opt_roi_out_roi_vol", None) is not None:
        cargs.extend([
            "-roi-out",
            params.get("opt_roi_out_roi_vol", None)
        ])
    if params.get("opt_fill_value_value", None) is not None:
        cargs.extend([
            "-fill-value",
            str(params.get("opt_fill_value_value", None))
        ])
    if params.get("opt_exact_limit_dist", None) is not None:
        cargs.extend([
            "-exact-limit",
            str(params.get("opt_exact_limit_dist", None))
        ])
    if params.get("opt_approx_limit_dist", None) is not None:
        cargs.extend([
            "-approx-limit",
            str(params.get("opt_approx_limit_dist", None))
        ])
    if params.get("opt_approx_neighborhood_num", None) is not None:
        cargs.extend([
            "-approx-neighborhood",
            str(params.get("opt_approx_neighborhood_num", None))
        ])
    if params.get("opt_winding_method", None) is not None:
        cargs.extend([
            "-winding",
            params.get("opt_winding_method", None)
        ])
    return cargs


def create_signed_distance_volume_outputs(
    params: CreateSignedDistanceVolumeParameters,
    execution: Execution,
) -> CreateSignedDistanceVolumeOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CreateSignedDistanceVolumeOutputs(
        root=execution.output_file("."),
        outvol=execution.output_file(params.get("outvol", None)),
        opt_roi_out_roi_vol=execution.output_file(params.get("opt_roi_out_roi_vol", None)) if (params.get("opt_roi_out_roi_vol") is not None) else None,
    )
    return ret


def create_signed_distance_volume_execute(
    params: CreateSignedDistanceVolumeParameters,
    runner: Runner | None = None,
) -> CreateSignedDistanceVolumeOutputs:
    """
    create-signed-distance-volume
    
    Create signed distance volume from surface.
    
    Computes the signed distance function of the surface. Exact distance is
    calculated by finding the closest point on any surface triangle to the
    center of the voxel. Approximate distance is calculated starting with these
    distances, using dijkstra's method with a neighborhood of voxels. Specifying
    too small of an exact distance may produce unexpected results. Valid
    specifiers for winding methods are as follows:
    
    EVEN_ODD (default)
    NEGATIVE
    NONZERO
    NORMALS
    
    The NORMALS method uses the normals of triangles and edges, or the closest
    triangle hit by a ray from the point. This method may be slightly faster,
    but is only reliable for a closed surface that does not cross through
    itself. All other methods count entry (positive) and exit (negative)
    crossings of a vertical ray from the point, then counts as inside if the
    total is odd, negative, or nonzero, respectively.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CreateSignedDistanceVolumeOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CREATE_SIGNED_DISTANCE_VOLUME_METADATA)
    params = execution.params(params)
    cargs = create_signed_distance_volume_cargs(params, execution)
    ret = create_signed_distance_volume_outputs(params, execution)
    execution.run(cargs)
    return ret


def create_signed_distance_volume(
    surface: InputPathType,
    refspace: str,
    outvol: str,
    opt_roi_out_roi_vol: str | None = None,
    opt_fill_value_value: float | None = None,
    opt_exact_limit_dist: float | None = None,
    opt_approx_limit_dist: float | None = None,
    opt_approx_neighborhood_num: int | None = None,
    opt_winding_method: str | None = None,
    runner: Runner | None = None,
) -> CreateSignedDistanceVolumeOutputs:
    """
    create-signed-distance-volume
    
    Create signed distance volume from surface.
    
    Computes the signed distance function of the surface. Exact distance is
    calculated by finding the closest point on any surface triangle to the
    center of the voxel. Approximate distance is calculated starting with these
    distances, using dijkstra's method with a neighborhood of voxels. Specifying
    too small of an exact distance may produce unexpected results. Valid
    specifiers for winding methods are as follows:
    
    EVEN_ODD (default)
    NEGATIVE
    NONZERO
    NORMALS
    
    The NORMALS method uses the normals of triangles and edges, or the closest
    triangle hit by a ray from the point. This method may be slightly faster,
    but is only reliable for a closed surface that does not cross through
    itself. All other methods count entry (positive) and exit (negative)
    crossings of a vertical ray from the point, then counts as inside if the
    total is odd, negative, or nonzero, respectively.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        surface: the input surface.
        refspace: a volume in the desired output space (dims, spacing, origin).
        outvol: the output volume.
        opt_roi_out_roi_vol: output an roi volume of where the output has a\
            computed value: the output roi volume.
        opt_fill_value_value: specify a value to put in all voxels that don't\
            get assigned a distance: value to fill with (default 0).
        opt_exact_limit_dist: specify distance for exact output: distance in mm\
            (default 5).
        opt_approx_limit_dist: specify distance for approximate output:\
            distance in mm (default 20).
        opt_approx_neighborhood_num: voxel neighborhood for approximate\
            calculation: size of neighborhood cube measured from center to face, in\
            voxels (default 2 = 5x5x5).
        opt_winding_method: winding method for point inside surface test: name\
            of the method (default EVEN_ODD).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CreateSignedDistanceVolumeOutputs`).
    """
    params = create_signed_distance_volume_params(
        surface=surface,
        refspace=refspace,
        outvol=outvol,
        opt_roi_out_roi_vol=opt_roi_out_roi_vol,
        opt_fill_value_value=opt_fill_value_value,
        opt_exact_limit_dist=opt_exact_limit_dist,
        opt_approx_limit_dist=opt_approx_limit_dist,
        opt_approx_neighborhood_num=opt_approx_neighborhood_num,
        opt_winding_method=opt_winding_method,
    )
    return create_signed_distance_volume_execute(params, runner)


__all__ = [
    "CREATE_SIGNED_DISTANCE_VOLUME_METADATA",
    "CreateSignedDistanceVolumeOutputs",
    "create_signed_distance_volume",
    "create_signed_distance_volume_execute",
    "create_signed_distance_volume_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

LABEL_MERGE_METADATA = Metadata(
    id="0ecf9e8d0dbbf764d34113468b73d18237947b26.boutiques",
    name="label-merge",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


LabelMergeUpToParameters = typing.TypedDict('LabelMergeUpToParameters', {
    "@type": typing.NotRequired[typing.Literal["up_to"]],
    "last_column": str,
    "opt_reverse": bool,
})
LabelMergeUpToParametersTagged = typing.TypedDict('LabelMergeUpToParametersTagged', {
    "@type": typing.Literal["up_to"],
    "last_column": str,
    "opt_reverse": bool,
})


LabelMergeColumnParameters = typing.TypedDict('LabelMergeColumnParameters', {
    "@type": typing.NotRequired[typing.Literal["column"]],
    "column": str,
    "up_to": typing.NotRequired[LabelMergeUpToParameters | None],
})
LabelMergeColumnParametersTagged = typing.TypedDict('LabelMergeColumnParametersTagged', {
    "@type": typing.Literal["column"],
    "column": str,
    "up_to": typing.NotRequired[LabelMergeUpToParameters | None],
})


LabelMergeLabelParameters = typing.TypedDict('LabelMergeLabelParameters', {
    "@type": typing.NotRequired[typing.Literal["label"]],
    "label_in": InputPathType,
    "column": typing.NotRequired[list[LabelMergeColumnParameters] | None],
})
LabelMergeLabelParametersTagged = typing.TypedDict('LabelMergeLabelParametersTagged', {
    "@type": typing.Literal["label"],
    "label_in": InputPathType,
    "column": typing.NotRequired[list[LabelMergeColumnParameters] | None],
})


LabelMergeParameters = typing.TypedDict('LabelMergeParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/label-merge"]],
    "label_out": str,
    "label": typing.NotRequired[list[LabelMergeLabelParameters] | None],
})
LabelMergeParametersTagged = typing.TypedDict('LabelMergeParametersTagged', {
    "@type": typing.Literal["workbench/label-merge"],
    "label_out": str,
    "label": typing.NotRequired[list[LabelMergeLabelParameters] | None],
})


def label_merge_up_to_params(
    last_column: str,
    opt_reverse: bool = False,
) -> LabelMergeUpToParametersTagged:
    """
    Build parameters.
    
    Args:
        last_column: the number or name of the last column to include.
        opt_reverse: use the range in reverse order.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "up_to",
        "last_column": last_column,
        "opt_reverse": opt_reverse,
    }
    return params


def label_merge_up_to_cargs(
    params: LabelMergeUpToParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-up-to")
    cargs.append(params.get("last_column", None))
    if params.get("opt_reverse", False):
        cargs.append("-reverse")
    return cargs


def label_merge_column_params(
    column: str,
    up_to: LabelMergeUpToParameters | None = None,
) -> LabelMergeColumnParametersTagged:
    """
    Build parameters.
    
    Args:
        column: the column number or name.
        up_to: use an inclusive range of columns.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "column",
        "column": column,
    }
    if up_to is not None:
        params["up_to"] = up_to
    return params


def label_merge_column_cargs(
    params: LabelMergeColumnParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-column")
    cargs.append(params.get("column", None))
    if params.get("up_to", None) is not None:
        cargs.extend(label_merge_up_to_cargs(params.get("up_to", None), execution))
    return cargs


def label_merge_label_params(
    label_in: InputPathType,
    column: list[LabelMergeColumnParameters] | None = None,
) -> LabelMergeLabelParametersTagged:
    """
    Build parameters.
    
    Args:
        label_in: a label file to use columns from.
        column: select a single column to use.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "label",
        "label_in": label_in,
    }
    if column is not None:
        params["column"] = column
    return params


def label_merge_label_cargs(
    params: LabelMergeLabelParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-label")
    cargs.append(execution.input_file(params.get("label_in", None)))
    if params.get("column", None) is not None:
        cargs.extend([a for c in [label_merge_column_cargs(s, execution) for s in params.get("column", None)] for a in c])
    return cargs


class LabelMergeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `LabelMergeParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    label_out: OutputPathType
    """the output label"""


def label_merge_params(
    label_out: str,
    label: list[LabelMergeLabelParameters] | None = None,
) -> LabelMergeParametersTagged:
    """
    Build parameters.
    
    Args:
        label_out: the output label.
        label: specify an input label.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/label-merge",
        "label_out": label_out,
    }
    if label is not None:
        params["label"] = label
    return params


def label_merge_cargs(
    params: LabelMergeParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-label-merge")
    cargs.append(params.get("label_out", None))
    if params.get("label", None) is not None:
        cargs.extend([a for c in [label_merge_label_cargs(s, execution) for s in params.get("label", None)] for a in c])
    return cargs


def label_merge_outputs(
    params: LabelMergeParameters,
    execution: Execution,
) -> LabelMergeOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = LabelMergeOutputs(
        root=execution.output_file("."),
        label_out=execution.output_file(params.get("label_out", None)),
    )
    return ret


def label_merge_execute(
    params: LabelMergeParameters,
    runner: Runner | None = None,
) -> LabelMergeOutputs:
    """
    label-merge
    
    Merge label files into a new file.
    
    Takes one or more label files and constructs a new label file by
    concatenating columns from them. The input files must have the same number
    of vertices and the same structure.
    
    Example: wb_command -label-merge out.label.gii -label first.label.gii
    -column 1 -label second.label.gii
    
    This example would take the first column from first.label.gii and all
    subvolumes from second.label.gii, and write these to out.label.gii.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelMergeOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(LABEL_MERGE_METADATA)
    params = execution.params(params)
    cargs = label_merge_cargs(params, execution)
    ret = label_merge_outputs(params, execution)
    execution.run(cargs)
    return ret


def label_merge(
    label_out: str,
    label: list[LabelMergeLabelParameters] | None = None,
    runner: Runner | None = None,
) -> LabelMergeOutputs:
    """
    label-merge
    
    Merge label files into a new file.
    
    Takes one or more label files and constructs a new label file by
    concatenating columns from them. The input files must have the same number
    of vertices and the same structure.
    
    Example: wb_command -label-merge out.label.gii -label first.label.gii
    -column 1 -label second.label.gii
    
    This example would take the first column from first.label.gii and all
    subvolumes from second.label.gii, and write these to out.label.gii.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        label_out: the output label.
        label: specify an input label.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelMergeOutputs`).
    """
    params = label_merge_params(
        label_out=label_out,
        label=label,
    )
    return label_merge_execute(params, runner)


__all__ = [
    "LABEL_MERGE_METADATA",
    "LabelMergeOutputs",
    "label_merge",
    "label_merge_column_params",
    "label_merge_execute",
    "label_merge_label_params",
    "label_merge_params",
    "label_merge_up_to_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_TO_SURFACE_MAPPING_METADATA = Metadata(
    id="677cb03d3aef10ca7afb17c714f6c3c6f70b3a93.boutiques",
    name="volume-to-surface-mapping",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


VolumeToSurfaceMappingVolumeRoiParameters = typing.TypedDict('VolumeToSurfaceMappingVolumeRoiParameters', {
    "@type": typing.NotRequired[typing.Literal["volume_roi"]],
    "roi_volume": InputPathType,
    "opt_weighted": bool,
})
VolumeToSurfaceMappingVolumeRoiParametersTagged = typing.TypedDict('VolumeToSurfaceMappingVolumeRoiParametersTagged', {
    "@type": typing.Literal["volume_roi"],
    "roi_volume": InputPathType,
    "opt_weighted": bool,
})


VolumeToSurfaceMappingOutputWeightsParameters = typing.TypedDict('VolumeToSurfaceMappingOutputWeightsParameters', {
    "@type": typing.NotRequired[typing.Literal["output_weights"]],
    "vertex": int,
    "weights_out": str,
})
VolumeToSurfaceMappingOutputWeightsParametersTagged = typing.TypedDict('VolumeToSurfaceMappingOutputWeightsParametersTagged', {
    "@type": typing.Literal["output_weights"],
    "vertex": int,
    "weights_out": str,
})


VolumeToSurfaceMappingRibbonConstrainedParameters = typing.TypedDict('VolumeToSurfaceMappingRibbonConstrainedParameters', {
    "@type": typing.NotRequired[typing.Literal["ribbon_constrained"]],
    "inner_surf": InputPathType,
    "outer_surf": InputPathType,
    "volume_roi": typing.NotRequired[VolumeToSurfaceMappingVolumeRoiParameters | None],
    "opt_voxel_subdiv_subdiv_num": typing.NotRequired[int | None],
    "opt_thin_columns": bool,
    "opt_gaussian_scale": typing.NotRequired[float | None],
    "opt_interpolate_method": typing.NotRequired[str | None],
    "opt_bad_vertices_out_roi_out": typing.NotRequired[str | None],
    "output_weights": typing.NotRequired[VolumeToSurfaceMappingOutputWeightsParameters | None],
    "opt_output_weights_text_text_out": typing.NotRequired[str | None],
})
VolumeToSurfaceMappingRibbonConstrainedParametersTagged = typing.TypedDict('VolumeToSurfaceMappingRibbonConstrainedParametersTagged', {
    "@type": typing.Literal["ribbon_constrained"],
    "inner_surf": InputPathType,
    "outer_surf": InputPathType,
    "volume_roi": typing.NotRequired[VolumeToSurfaceMappingVolumeRoiParameters | None],
    "opt_voxel_subdiv_subdiv_num": typing.NotRequired[int | None],
    "opt_thin_columns": bool,
    "opt_gaussian_scale": typing.NotRequired[float | None],
    "opt_interpolate_method": typing.NotRequired[str | None],
    "opt_bad_vertices_out_roi_out": typing.NotRequired[str | None],
    "output_weights": typing.NotRequired[VolumeToSurfaceMappingOutputWeightsParameters | None],
    "opt_output_weights_text_text_out": typing.NotRequired[str | None],
})


VolumeToSurfaceMappingMyelinStyleParameters = typing.TypedDict('VolumeToSurfaceMappingMyelinStyleParameters', {
    "@type": typing.NotRequired[typing.Literal["myelin_style"]],
    "ribbon_roi": InputPathType,
    "thickness": InputPathType,
    "sigma": float,
    "opt_legacy_bug": bool,
})
VolumeToSurfaceMappingMyelinStyleParametersTagged = typing.TypedDict('VolumeToSurfaceMappingMyelinStyleParametersTagged', {
    "@type": typing.Literal["myelin_style"],
    "ribbon_roi": InputPathType,
    "thickness": InputPathType,
    "sigma": float,
    "opt_legacy_bug": bool,
})


VolumeToSurfaceMappingParameters = typing.TypedDict('VolumeToSurfaceMappingParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/volume-to-surface-mapping"]],
    "volume": InputPathType,
    "surface": InputPathType,
    "metric_out": str,
    "opt_trilinear": bool,
    "opt_enclosing": bool,
    "opt_cubic": bool,
    "ribbon_constrained": typing.NotRequired[VolumeToSurfaceMappingRibbonConstrainedParameters | None],
    "myelin_style": typing.NotRequired[VolumeToSurfaceMappingMyelinStyleParameters | None],
    "opt_subvol_select_subvol": typing.NotRequired[str | None],
})
VolumeToSurfaceMappingParametersTagged = typing.TypedDict('VolumeToSurfaceMappingParametersTagged', {
    "@type": typing.Literal["workbench/volume-to-surface-mapping"],
    "volume": InputPathType,
    "surface": InputPathType,
    "metric_out": str,
    "opt_trilinear": bool,
    "opt_enclosing": bool,
    "opt_cubic": bool,
    "ribbon_constrained": typing.NotRequired[VolumeToSurfaceMappingRibbonConstrainedParameters | None],
    "myelin_style": typing.NotRequired[VolumeToSurfaceMappingMyelinStyleParameters | None],
    "opt_subvol_select_subvol": typing.NotRequired[str | None],
})


def volume_to_surface_mapping_volume_roi_params(
    roi_volume: InputPathType,
    opt_weighted: bool = False,
) -> VolumeToSurfaceMappingVolumeRoiParametersTagged:
    """
    Build parameters.
    
    Args:
        roi_volume: the roi volume file.
        opt_weighted: treat the roi values as weightings rather than binary.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "volume_roi",
        "roi_volume": roi_volume,
        "opt_weighted": opt_weighted,
    }
    return params


def volume_to_surface_mapping_volume_roi_cargs(
    params: VolumeToSurfaceMappingVolumeRoiParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-volume-roi")
    cargs.append(execution.input_file(params.get("roi_volume", None)))
    if params.get("opt_weighted", False):
        cargs.append("-weighted")
    return cargs


class VolumeToSurfaceMappingOutputWeightsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeToSurfaceMappingOutputWeightsParameters | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    weights_out: OutputPathType
    """volume to write the weights to"""


def volume_to_surface_mapping_output_weights_params(
    vertex: int,
    weights_out: str,
) -> VolumeToSurfaceMappingOutputWeightsParametersTagged:
    """
    Build parameters.
    
    Args:
        vertex: the vertex number to get the voxel weights for, 0-based.
        weights_out: volume to write the weights to.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "output_weights",
        "vertex": vertex,
        "weights_out": weights_out,
    }
    return params


def volume_to_surface_mapping_output_weights_cargs(
    params: VolumeToSurfaceMappingOutputWeightsParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-output-weights")
    cargs.append(str(params.get("vertex", None)))
    cargs.append(params.get("weights_out", None))
    return cargs


def volume_to_surface_mapping_output_weights_outputs(
    params: VolumeToSurfaceMappingOutputWeightsParameters,
    execution: Execution,
) -> VolumeToSurfaceMappingOutputWeightsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeToSurfaceMappingOutputWeightsOutputs(
        root=execution.output_file("."),
        weights_out=execution.output_file(params.get("weights_out", None)),
    )
    return ret


class VolumeToSurfaceMappingRibbonConstrainedOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeToSurfaceMappingRibbonConstrainedParameters | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_weights: VolumeToSurfaceMappingOutputWeightsOutputs | None
    """Outputs from `volume_to_surface_mapping_output_weights_outputs`."""


def volume_to_surface_mapping_ribbon_constrained_params(
    inner_surf: InputPathType,
    outer_surf: InputPathType,
    volume_roi: VolumeToSurfaceMappingVolumeRoiParameters | None = None,
    opt_voxel_subdiv_subdiv_num: int | None = None,
    opt_thin_columns: bool = False,
    opt_gaussian_scale: float | None = None,
    opt_interpolate_method: str | None = None,
    opt_bad_vertices_out_roi_out: str | None = None,
    output_weights: VolumeToSurfaceMappingOutputWeightsParameters | None = None,
    opt_output_weights_text_text_out: str | None = None,
) -> VolumeToSurfaceMappingRibbonConstrainedParametersTagged:
    """
    Build parameters.
    
    Args:
        inner_surf: the inner surface of the ribbon.
        outer_surf: the outer surface of the ribbon.
        volume_roi: use a volume roi.
        opt_voxel_subdiv_subdiv_num: voxel divisions while estimating voxel\
            weights: number of subdivisions, default 3.
        opt_thin_columns: use non-overlapping polyhedra.
        opt_gaussian_scale: reduce weight to voxels that aren't near <surface>:\
            value to multiply the local thickness by, to get the gaussian sigma.
        opt_interpolate_method: instead of a weighted average of voxels,\
            interpolate at subpoints inside the ribbon: interpolation method, must\
            be CUBIC, ENCLOSING_VOXEL, or TRILINEAR.
        opt_bad_vertices_out_roi_out: output an ROI of which vertices didn't\
            intersect any valid voxels: the output metric file of vertices that\
            have no data.
        output_weights: write the voxel weights for a vertex to a volume file.
        opt_output_weights_text_text_out: write the voxel weights for all\
            vertices to a text file: output - the output text filename.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ribbon_constrained",
        "inner_surf": inner_surf,
        "outer_surf": outer_surf,
        "opt_thin_columns": opt_thin_columns,
    }
    if volume_roi is not None:
        params["volume_roi"] = volume_roi
    if opt_voxel_subdiv_subdiv_num is not None:
        params["opt_voxel_subdiv_subdiv_num"] = opt_voxel_subdiv_subdiv_num
    if opt_gaussian_scale is not None:
        params["opt_gaussian_scale"] = opt_gaussian_scale
    if opt_interpolate_method is not None:
        params["opt_interpolate_method"] = opt_interpolate_method
    if opt_bad_vertices_out_roi_out is not None:
        params["opt_bad_vertices_out_roi_out"] = opt_bad_vertices_out_roi_out
    if output_weights is not None:
        params["output_weights"] = output_weights
    if opt_output_weights_text_text_out is not None:
        params["opt_output_weights_text_text_out"] = opt_output_weights_text_text_out
    return params


def volume_to_surface_mapping_ribbon_constrained_cargs(
    params: VolumeToSurfaceMappingRibbonConstrainedParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-ribbon-constrained")
    cargs.append(execution.input_file(params.get("inner_surf", None)))
    cargs.append(execution.input_file(params.get("outer_surf", None)))
    if params.get("volume_roi", None) is not None:
        cargs.extend(volume_to_surface_mapping_volume_roi_cargs(params.get("volume_roi", None), execution))
    if params.get("opt_voxel_subdiv_subdiv_num", None) is not None:
        cargs.extend([
            "-voxel-subdiv",
            str(params.get("opt_voxel_subdiv_subdiv_num", None))
        ])
    if params.get("opt_thin_columns", False):
        cargs.append("-thin-columns")
    if params.get("opt_gaussian_scale", None) is not None:
        cargs.extend([
            "-gaussian",
            str(params.get("opt_gaussian_scale", None))
        ])
    if params.get("opt_interpolate_method", None) is not None:
        cargs.extend([
            "-interpolate",
            params.get("opt_interpolate_method", None)
        ])
    if params.get("opt_bad_vertices_out_roi_out", None) is not None:
        cargs.extend([
            "-bad-vertices-out",
            params.get("opt_bad_vertices_out_roi_out", None)
        ])
    if params.get("output_weights", None) is not None:
        cargs.extend(volume_to_surface_mapping_output_weights_cargs(params.get("output_weights", None), execution))
    if params.get("opt_output_weights_text_text_out", None) is not None:
        cargs.extend([
            "-output-weights-text",
            params.get("opt_output_weights_text_text_out", None)
        ])
    return cargs


def volume_to_surface_mapping_ribbon_constrained_outputs(
    params: VolumeToSurfaceMappingRibbonConstrainedParameters,
    execution: Execution,
) -> VolumeToSurfaceMappingRibbonConstrainedOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeToSurfaceMappingRibbonConstrainedOutputs(
        root=execution.output_file("."),
        output_weights=volume_to_surface_mapping_output_weights_outputs(params.get("output_weights"), execution) if params.get("output_weights") else None,
    )
    return ret


def volume_to_surface_mapping_myelin_style_params(
    ribbon_roi: InputPathType,
    thickness: InputPathType,
    sigma: float,
    opt_legacy_bug: bool = False,
) -> VolumeToSurfaceMappingMyelinStyleParametersTagged:
    """
    Build parameters.
    
    Args:
        ribbon_roi: an roi volume of the cortical ribbon for this hemisphere.
        thickness: a metric file of cortical thickness.
        sigma: gaussian kernel in mm for weighting voxels within range.
        opt_legacy_bug: emulate old v1.2.3 and earlier code that didn't follow\
            a cylinder cutoff.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "myelin_style",
        "ribbon_roi": ribbon_roi,
        "thickness": thickness,
        "sigma": sigma,
        "opt_legacy_bug": opt_legacy_bug,
    }
    return params


def volume_to_surface_mapping_myelin_style_cargs(
    params: VolumeToSurfaceMappingMyelinStyleParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-myelin-style")
    cargs.append(execution.input_file(params.get("ribbon_roi", None)))
    cargs.append(execution.input_file(params.get("thickness", None)))
    cargs.append(str(params.get("sigma", None)))
    if params.get("opt_legacy_bug", False):
        cargs.append("-legacy-bug")
    return cargs


class VolumeToSurfaceMappingOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeToSurfaceMappingParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output metric file"""
    ribbon_constrained: VolumeToSurfaceMappingRibbonConstrainedOutputs | None
    """Outputs from `volume_to_surface_mapping_ribbon_constrained_outputs`."""


def volume_to_surface_mapping_params(
    volume: InputPathType,
    surface: InputPathType,
    metric_out: str,
    opt_trilinear: bool = False,
    opt_enclosing: bool = False,
    opt_cubic: bool = False,
    ribbon_constrained: VolumeToSurfaceMappingRibbonConstrainedParameters | None = None,
    myelin_style: VolumeToSurfaceMappingMyelinStyleParameters | None = None,
    opt_subvol_select_subvol: str | None = None,
) -> VolumeToSurfaceMappingParametersTagged:
    """
    Build parameters.
    
    Args:
        volume: the volume to map data from.
        surface: the surface to map the data onto.
        metric_out: the output metric file.
        opt_trilinear: use trilinear volume interpolation.
        opt_enclosing: use value of the enclosing voxel.
        opt_cubic: use cubic splines.
        ribbon_constrained: use ribbon constrained mapping algorithm.
        myelin_style: use the method from myelin mapping.
        opt_subvol_select_subvol: select a single subvolume to map: the\
            subvolume number or name.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-to-surface-mapping",
        "volume": volume,
        "surface": surface,
        "metric_out": metric_out,
        "opt_trilinear": opt_trilinear,
        "opt_enclosing": opt_enclosing,
        "opt_cubic": opt_cubic,
    }
    if ribbon_constrained is not None:
        params["ribbon_constrained"] = ribbon_constrained
    if myelin_style is not None:
        params["myelin_style"] = myelin_style
    if opt_subvol_select_subvol is not None:
        params["opt_subvol_select_subvol"] = opt_subvol_select_subvol
    return params


def volume_to_surface_mapping_cargs(
    params: VolumeToSurfaceMappingParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-volume-to-surface-mapping")
    cargs.append(execution.input_file(params.get("volume", None)))
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(params.get("metric_out", None))
    if params.get("opt_trilinear", False):
        cargs.append("-trilinear")
    if params.get("opt_enclosing", False):
        cargs.append("-enclosing")
    if params.get("opt_cubic", False):
        cargs.append("-cubic")
    if params.get("ribbon_constrained", None) is not None:
        cargs.extend(volume_to_surface_mapping_ribbon_constrained_cargs(params.get("ribbon_constrained", None), execution))
    if params.get("myelin_style", None) is not None:
        cargs.extend(volume_to_surface_mapping_myelin_style_cargs(params.get("myelin_style", None), execution))
    if params.get("opt_subvol_select_subvol", None) is not None:
        cargs.extend([
            "-subvol-select",
            params.get("opt_subvol_select_subvol", None)
        ])
    return cargs


def volume_to_surface_mapping_outputs(
    params: VolumeToSurfaceMappingParameters,
    execution: Execution,
) -> VolumeToSurfaceMappingOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeToSurfaceMappingOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric_out", None)),
        ribbon_constrained=volume_to_surface_mapping_ribbon_constrained_outputs(params.get("ribbon_constrained"), execution) if params.get("ribbon_constrained") else None,
    )
    return ret


def volume_to_surface_mapping_execute(
    params: VolumeToSurfaceMappingParameters,
    runner: Runner | None = None,
) -> VolumeToSurfaceMappingOutputs:
    """
    volume-to-surface-mapping
    
    Map volume to surface.
    
    You must specify exactly one mapping method. Enclosing voxel uses the value
    from the voxel the vertex lies inside, while trilinear does a 3D linear
    interpolation based on the voxels immediately on each side of the vertex's
    position.
    
    The ribbon mapping method constructs a polyhedron from the vertex's
    neighbors on each surface, and estimates the amount of this polyhedron's
    volume that falls inside any nearby voxels, to use as the weights for
    sampling. If -thin-columns is specified, the polyhedron uses the edge
    midpoints and triangle centroids, so that neighboring vertices do not have
    overlapping polyhedra. This may require increasing -voxel-subdiv to get
    enough samples in each voxel to reliably land inside these smaller
    polyhedra. The volume ROI is useful to exclude partial volume effects of
    voxels the surfaces pass through, and will cause the mapping to ignore
    voxels that don't have a positive value in the mask. The subdivision number
    specifies how it approximates the amount of the volume the polyhedron
    intersects, by splitting each voxel into NxNxN pieces, and checking whether
    the center of each piece is inside the polyhedron. If you have very large
    voxels, consider increasing this if you get zeros in your output. The
    -gaussian option makes it act more like the myelin method, where the
    distance of a voxel from <surface> is used to downweight the voxel. The
    -interpolate suboption, instead of doing a weighted average of voxels,
    interpolates from the volume at the subdivided points inside the ribbon. If
    using both -interpolate and the -weighted suboption to -volume-roi, the roi
    volume weights are linearly interpolated, unless the -interpolate method is
    ENCLOSING_VOXEL, in which case ENCLOSING_VOXEL is also used for sampling the
    roi volume weights.
    
    The myelin style method uses part of the caret5 myelin mapping command to do
    the mapping: for each surface vertex, take all voxels that are in a cylinder
    with radius and height equal to cortical thickness, centered on the vertex
    and aligned with the surface normal, and that are also within the ribbon
    ROI, and apply a gaussian kernel with the specified sigma to them to get the
    weights to use. The -legacy-bug flag reverts to the unintended behavior
    present from the initial implementation up to and including v1.2.3, which
    had only the tangential cutoff and a bounding box intended to be larger than
    where the cylinder cutoff should have been.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeToSurfaceMappingOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_TO_SURFACE_MAPPING_METADATA)
    params = execution.params(params)
    cargs = volume_to_surface_mapping_cargs(params, execution)
    ret = volume_to_surface_mapping_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_to_surface_mapping(
    volume: InputPathType,
    surface: InputPathType,
    metric_out: str,
    opt_trilinear: bool = False,
    opt_enclosing: bool = False,
    opt_cubic: bool = False,
    ribbon_constrained: VolumeToSurfaceMappingRibbonConstrainedParameters | None = None,
    myelin_style: VolumeToSurfaceMappingMyelinStyleParameters | None = None,
    opt_subvol_select_subvol: str | None = None,
    runner: Runner | None = None,
) -> VolumeToSurfaceMappingOutputs:
    """
    volume-to-surface-mapping
    
    Map volume to surface.
    
    You must specify exactly one mapping method. Enclosing voxel uses the value
    from the voxel the vertex lies inside, while trilinear does a 3D linear
    interpolation based on the voxels immediately on each side of the vertex's
    position.
    
    The ribbon mapping method constructs a polyhedron from the vertex's
    neighbors on each surface, and estimates the amount of this polyhedron's
    volume that falls inside any nearby voxels, to use as the weights for
    sampling. If -thin-columns is specified, the polyhedron uses the edge
    midpoints and triangle centroids, so that neighboring vertices do not have
    overlapping polyhedra. This may require increasing -voxel-subdiv to get
    enough samples in each voxel to reliably land inside these smaller
    polyhedra. The volume ROI is useful to exclude partial volume effects of
    voxels the surfaces pass through, and will cause the mapping to ignore
    voxels that don't have a positive value in the mask. The subdivision number
    specifies how it approximates the amount of the volume the polyhedron
    intersects, by splitting each voxel into NxNxN pieces, and checking whether
    the center of each piece is inside the polyhedron. If you have very large
    voxels, consider increasing this if you get zeros in your output. The
    -gaussian option makes it act more like the myelin method, where the
    distance of a voxel from <surface> is used to downweight the voxel. The
    -interpolate suboption, instead of doing a weighted average of voxels,
    interpolates from the volume at the subdivided points inside the ribbon. If
    using both -interpolate and the -weighted suboption to -volume-roi, the roi
    volume weights are linearly interpolated, unless the -interpolate method is
    ENCLOSING_VOXEL, in which case ENCLOSING_VOXEL is also used for sampling the
    roi volume weights.
    
    The myelin style method uses part of the caret5 myelin mapping command to do
    the mapping: for each surface vertex, take all voxels that are in a cylinder
    with radius and height equal to cortical thickness, centered on the vertex
    and aligned with the surface normal, and that are also within the ribbon
    ROI, and apply a gaussian kernel with the specified sigma to them to get the
    weights to use. The -legacy-bug flag reverts to the unintended behavior
    present from the initial implementation up to and including v1.2.3, which
    had only the tangential cutoff and a bounding box intended to be larger than
    where the cylinder cutoff should have been.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        volume: the volume to map data from.
        surface: the surface to map the data onto.
        metric_out: the output metric file.
        opt_trilinear: use trilinear volume interpolation.
        opt_enclosing: use value of the enclosing voxel.
        opt_cubic: use cubic splines.
        ribbon_constrained: use ribbon constrained mapping algorithm.
        myelin_style: use the method from myelin mapping.
        opt_subvol_select_subvol: select a single subvolume to map: the\
            subvolume number or name.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeToSurfaceMappingOutputs`).
    """
    params = volume_to_surface_mapping_params(
        volume=volume,
        surface=surface,
        metric_out=metric_out,
        opt_trilinear=opt_trilinear,
        opt_enclosing=opt_enclosing,
        opt_cubic=opt_cubic,
        ribbon_constrained=ribbon_constrained,
        myelin_style=myelin_style,
        opt_subvol_select_subvol=opt_subvol_select_subvol,
    )
    return volume_to_surface_mapping_execute(params, runner)


__all__ = [
    "VOLUME_TO_SURFACE_MAPPING_METADATA",
    "VolumeToSurfaceMappingOutputWeightsOutputs",
    "VolumeToSurfaceMappingOutputs",
    "VolumeToSurfaceMappingRibbonConstrainedOutputs",
    "volume_to_surface_mapping",
    "volume_to_surface_mapping_execute",
    "volume_to_surface_mapping_myelin_style_params",
    "volume_to_surface_mapping_output_weights_params",
    "volume_to_surface_mapping_params",
    "volume_to_surface_mapping_ribbon_constrained_params",
    "volume_to_surface_mapping_volume_roi_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CONVERT_WARPFIELD_METADATA = Metadata(
    id="5fe37befef070c4bb946473c38eed5fa3b83614b.boutiques",
    name="convert-warpfield",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


ConvertWarpfieldFromWorldParameters = typing.TypedDict('ConvertWarpfieldFromWorldParameters', {
    "@type": typing.NotRequired[typing.Literal["from_world"]],
    "input": str,
    "opt_absolute": bool,
})
ConvertWarpfieldFromWorldParametersTagged = typing.TypedDict('ConvertWarpfieldFromWorldParametersTagged', {
    "@type": typing.Literal["from_world"],
    "input": str,
    "opt_absolute": bool,
})


ConvertWarpfieldFromFnirtParameters = typing.TypedDict('ConvertWarpfieldFromFnirtParameters', {
    "@type": typing.NotRequired[typing.Literal["from_fnirt"]],
    "input": str,
    "source_volume": str,
    "opt_absolute": bool,
})
ConvertWarpfieldFromFnirtParametersTagged = typing.TypedDict('ConvertWarpfieldFromFnirtParametersTagged', {
    "@type": typing.Literal["from_fnirt"],
    "input": str,
    "source_volume": str,
    "opt_absolute": bool,
})


ConvertWarpfieldToFnirtParameters = typing.TypedDict('ConvertWarpfieldToFnirtParameters', {
    "@type": typing.NotRequired[typing.Literal["to_fnirt"]],
    "output": str,
    "source_volume": str,
})
ConvertWarpfieldToFnirtParametersTagged = typing.TypedDict('ConvertWarpfieldToFnirtParametersTagged', {
    "@type": typing.Literal["to_fnirt"],
    "output": str,
    "source_volume": str,
})


ConvertWarpfieldParameters = typing.TypedDict('ConvertWarpfieldParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/convert-warpfield"]],
    "from_world": typing.NotRequired[ConvertWarpfieldFromWorldParameters | None],
    "opt_from_itk_input": typing.NotRequired[str | None],
    "from_fnirt": typing.NotRequired[ConvertWarpfieldFromFnirtParameters | None],
    "opt_to_world_output": typing.NotRequired[str | None],
    "opt_to_itk_output": typing.NotRequired[str | None],
    "to_fnirt": typing.NotRequired[list[ConvertWarpfieldToFnirtParameters] | None],
})
ConvertWarpfieldParametersTagged = typing.TypedDict('ConvertWarpfieldParametersTagged', {
    "@type": typing.Literal["workbench/convert-warpfield"],
    "from_world": typing.NotRequired[ConvertWarpfieldFromWorldParameters | None],
    "opt_from_itk_input": typing.NotRequired[str | None],
    "from_fnirt": typing.NotRequired[ConvertWarpfieldFromFnirtParameters | None],
    "opt_to_world_output": typing.NotRequired[str | None],
    "opt_to_itk_output": typing.NotRequired[str | None],
    "to_fnirt": typing.NotRequired[list[ConvertWarpfieldToFnirtParameters] | None],
})


def convert_warpfield_from_world_params(
    input_: str,
    opt_absolute: bool = False,
) -> ConvertWarpfieldFromWorldParametersTagged:
    """
    Build parameters.
    
    Args:
        input_: the input warpfield.
        opt_absolute: warpfield was written in absolute convention, rather than\
            relative.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "from_world",
        "input": input_,
        "opt_absolute": opt_absolute,
    }
    return params


def convert_warpfield_from_world_cargs(
    params: ConvertWarpfieldFromWorldParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-from-world")
    cargs.append(params.get("input", None))
    if params.get("opt_absolute", False):
        cargs.append("-absolute")
    return cargs


def convert_warpfield_from_fnirt_params(
    input_: str,
    source_volume: str,
    opt_absolute: bool = False,
) -> ConvertWarpfieldFromFnirtParametersTagged:
    """
    Build parameters.
    
    Args:
        input_: the input warpfield.
        source_volume: the source volume used when generating the input\
            warpfield.
        opt_absolute: warpfield was written in absolute convention, rather than\
            relative.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "from_fnirt",
        "input": input_,
        "source_volume": source_volume,
        "opt_absolute": opt_absolute,
    }
    return params


def convert_warpfield_from_fnirt_cargs(
    params: ConvertWarpfieldFromFnirtParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-from-fnirt")
    cargs.append(params.get("input", None))
    cargs.append(params.get("source_volume", None))
    if params.get("opt_absolute", False):
        cargs.append("-absolute")
    return cargs


def convert_warpfield_to_fnirt_params(
    output: str,
    source_volume: str,
) -> ConvertWarpfieldToFnirtParametersTagged:
    """
    Build parameters.
    
    Args:
        output: output - the output warpfield.
        source_volume: the volume you want to apply the warpfield to.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "to_fnirt",
        "output": output,
        "source_volume": source_volume,
    }
    return params


def convert_warpfield_to_fnirt_cargs(
    params: ConvertWarpfieldToFnirtParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-to-fnirt")
    cargs.append(params.get("output", None))
    cargs.append(params.get("source_volume", None))
    return cargs


class ConvertWarpfieldOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ConvertWarpfieldParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def convert_warpfield_params(
    from_world: ConvertWarpfieldFromWorldParameters | None = None,
    opt_from_itk_input: str | None = None,
    from_fnirt: ConvertWarpfieldFromFnirtParameters | None = None,
    opt_to_world_output: str | None = None,
    opt_to_itk_output: str | None = None,
    to_fnirt: list[ConvertWarpfieldToFnirtParameters] | None = None,
) -> ConvertWarpfieldParametersTagged:
    """
    Build parameters.
    
    Args:
        from_world: input is a NIFTI 'world' warpfield.
        opt_from_itk_input: input is an ITK warpfield: the input warpfield.
        from_fnirt: input is a fnirt warpfield.
        opt_to_world_output: write output as a NIFTI 'world' warpfield: output\
            - the output warpfield.
        opt_to_itk_output: write output as an ITK warpfield: output - the\
            output warpfield.
        to_fnirt: write output as a fnirt warpfield.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/convert-warpfield",
    }
    if from_world is not None:
        params["from_world"] = from_world
    if opt_from_itk_input is not None:
        params["opt_from_itk_input"] = opt_from_itk_input
    if from_fnirt is not None:
        params["from_fnirt"] = from_fnirt
    if opt_to_world_output is not None:
        params["opt_to_world_output"] = opt_to_world_output
    if opt_to_itk_output is not None:
        params["opt_to_itk_output"] = opt_to_itk_output
    if to_fnirt is not None:
        params["to_fnirt"] = to_fnirt
    return params


def convert_warpfield_cargs(
    params: ConvertWarpfieldParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-convert-warpfield")
    if params.get("from_world", None) is not None:
        cargs.extend(convert_warpfield_from_world_cargs(params.get("from_world", None), execution))
    if params.get("opt_from_itk_input", None) is not None:
        cargs.extend([
            "-from-itk",
            params.get("opt_from_itk_input", None)
        ])
    if params.get("from_fnirt", None) is not None:
        cargs.extend(convert_warpfield_from_fnirt_cargs(params.get("from_fnirt", None), execution))
    if params.get("opt_to_world_output", None) is not None:
        cargs.extend([
            "-to-world",
            params.get("opt_to_world_output", None)
        ])
    if params.get("opt_to_itk_output", None) is not None:
        cargs.extend([
            "-to-itk",
            params.get("opt_to_itk_output", None)
        ])
    if params.get("to_fnirt", None) is not None:
        cargs.extend([a for c in [convert_warpfield_to_fnirt_cargs(s, execution) for s in params.get("to_fnirt", None)] for a in c])
    return cargs


def convert_warpfield_outputs(
    params: ConvertWarpfieldParameters,
    execution: Execution,
) -> ConvertWarpfieldOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = ConvertWarpfieldOutputs(
        root=execution.output_file("."),
    )
    return ret


def convert_warpfield_execute(
    params: ConvertWarpfieldParameters,
    runner: Runner | None = None,
) -> ConvertWarpfieldOutputs:
    """
    convert-warpfield
    
    Convert a warpfield between conventions.
    
    NIFTI world warpfields can be used directly on mm coordinates via sampling
    the three subvolumes at the coordinate and adding the sampled values to the
    coordinate vector. They use the NIFTI coordinate system, that is, X is left
    to right, Y is posterior to anterior, and Z is inferior to superior.
    
    NOTE: this command does not invert the warpfield, and to warp a surface, you
    must use the inverse of the warpfield that warps the corresponding volume.
    
    The ITK format is used by ANTS.
    
    You must specify exactly one -from option, but you may specify multiple -to
    options, and -to-fnirt may be specified more than once.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ConvertWarpfieldOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CONVERT_WARPFIELD_METADATA)
    params = execution.params(params)
    cargs = convert_warpfield_cargs(params, execution)
    ret = convert_warpfield_outputs(params, execution)
    execution.run(cargs)
    return ret


def convert_warpfield(
    from_world: ConvertWarpfieldFromWorldParameters | None = None,
    opt_from_itk_input: str | None = None,
    from_fnirt: ConvertWarpfieldFromFnirtParameters | None = None,
    opt_to_world_output: str | None = None,
    opt_to_itk_output: str | None = None,
    to_fnirt: list[ConvertWarpfieldToFnirtParameters] | None = None,
    runner: Runner | None = None,
) -> ConvertWarpfieldOutputs:
    """
    convert-warpfield
    
    Convert a warpfield between conventions.
    
    NIFTI world warpfields can be used directly on mm coordinates via sampling
    the three subvolumes at the coordinate and adding the sampled values to the
    coordinate vector. They use the NIFTI coordinate system, that is, X is left
    to right, Y is posterior to anterior, and Z is inferior to superior.
    
    NOTE: this command does not invert the warpfield, and to warp a surface, you
    must use the inverse of the warpfield that warps the corresponding volume.
    
    The ITK format is used by ANTS.
    
    You must specify exactly one -from option, but you may specify multiple -to
    options, and -to-fnirt may be specified more than once.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        from_world: input is a NIFTI 'world' warpfield.
        opt_from_itk_input: input is an ITK warpfield: the input warpfield.
        from_fnirt: input is a fnirt warpfield.
        opt_to_world_output: write output as a NIFTI 'world' warpfield: output\
            - the output warpfield.
        opt_to_itk_output: write output as an ITK warpfield: output - the\
            output warpfield.
        to_fnirt: write output as a fnirt warpfield.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ConvertWarpfieldOutputs`).
    """
    params = convert_warpfield_params(
        from_world=from_world,
        opt_from_itk_input=opt_from_itk_input,
        from_fnirt=from_fnirt,
        opt_to_world_output=opt_to_world_output,
        opt_to_itk_output=opt_to_itk_output,
        to_fnirt=to_fnirt,
    )
    return convert_warpfield_execute(params, runner)


__all__ = [
    "CONVERT_WARPFIELD_METADATA",
    "ConvertWarpfieldOutputs",
    "convert_warpfield",
    "convert_warpfield_execute",
    "convert_warpfield_from_fnirt_params",
    "convert_warpfield_from_world_params",
    "convert_warpfield_params",
    "convert_warpfield_to_fnirt_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

METRIC_DILATE_METADATA = Metadata(
    id="228bae2c95e281c1ed3d0fb17aed811d10a80e3b.boutiques",
    name="metric-dilate",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


MetricDilateParameters = typing.TypedDict('MetricDilateParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/metric-dilate"]],
    "metric": InputPathType,
    "surface": InputPathType,
    "distance": float,
    "metric_out": str,
    "opt_bad_vertex_roi_roi_metric": typing.NotRequired[InputPathType | None],
    "opt_data_roi_roi_metric": typing.NotRequired[InputPathType | None],
    "opt_column_column": typing.NotRequired[str | None],
    "opt_nearest": bool,
    "opt_linear": bool,
    "opt_exponent_exponent": typing.NotRequired[float | None],
    "opt_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
    "opt_legacy_cutoff": bool,
})
MetricDilateParametersTagged = typing.TypedDict('MetricDilateParametersTagged', {
    "@type": typing.Literal["workbench/metric-dilate"],
    "metric": InputPathType,
    "surface": InputPathType,
    "distance": float,
    "metric_out": str,
    "opt_bad_vertex_roi_roi_metric": typing.NotRequired[InputPathType | None],
    "opt_data_roi_roi_metric": typing.NotRequired[InputPathType | None],
    "opt_column_column": typing.NotRequired[str | None],
    "opt_nearest": bool,
    "opt_linear": bool,
    "opt_exponent_exponent": typing.NotRequired[float | None],
    "opt_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
    "opt_legacy_cutoff": bool,
})


class MetricDilateOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetricDilateParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output metric"""


def metric_dilate_params(
    metric: InputPathType,
    surface: InputPathType,
    distance: float,
    metric_out: str,
    opt_bad_vertex_roi_roi_metric: InputPathType | None = None,
    opt_data_roi_roi_metric: InputPathType | None = None,
    opt_column_column: str | None = None,
    opt_nearest: bool = False,
    opt_linear: bool = False,
    opt_exponent_exponent: float | None = None,
    opt_corrected_areas_area_metric: InputPathType | None = None,
    opt_legacy_cutoff: bool = False,
) -> MetricDilateParametersTagged:
    """
    Build parameters.
    
    Args:
        metric: the metric to dilate.
        surface: the surface to compute on.
        distance: distance in mm to dilate.
        metric_out: the output metric.
        opt_bad_vertex_roi_roi_metric: specify an roi of vertices to overwrite,\
            rather than vertices with value zero: metric file, positive values\
            denote vertices to have their values replaced.
        opt_data_roi_roi_metric: specify an roi of where there is data: metric\
            file, positive values denote vertices that have data.
        opt_column_column: select a single column to dilate: the column number\
            or name.
        opt_nearest: use the nearest good value instead of a weighted average.
        opt_linear: fill in values with linear interpolation along strongest\
            gradient.
        opt_exponent_exponent: use a different exponent in the weighting\
            function: exponent 'n' to use in (area / (distance ^ n)) as the\
            weighting function (default 6).
        opt_corrected_areas_area_metric: vertex areas to use instead of\
            computing them from the surface: the corrected vertex areas, as a\
            metric.
        opt_legacy_cutoff: use the v1.3.2 method of choosing how many vertices\
            to use when calulating the dilated value with weighted method.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/metric-dilate",
        "metric": metric,
        "surface": surface,
        "distance": distance,
        "metric_out": metric_out,
        "opt_nearest": opt_nearest,
        "opt_linear": opt_linear,
        "opt_legacy_cutoff": opt_legacy_cutoff,
    }
    if opt_bad_vertex_roi_roi_metric is not None:
        params["opt_bad_vertex_roi_roi_metric"] = opt_bad_vertex_roi_roi_metric
    if opt_data_roi_roi_metric is not None:
        params["opt_data_roi_roi_metric"] = opt_data_roi_roi_metric
    if opt_column_column is not None:
        params["opt_column_column"] = opt_column_column
    if opt_exponent_exponent is not None:
        params["opt_exponent_exponent"] = opt_exponent_exponent
    if opt_corrected_areas_area_metric is not None:
        params["opt_corrected_areas_area_metric"] = opt_corrected_areas_area_metric
    return params


def metric_dilate_cargs(
    params: MetricDilateParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-metric-dilate")
    cargs.append(execution.input_file(params.get("metric", None)))
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(str(params.get("distance", None)))
    cargs.append(params.get("metric_out", None))
    if params.get("opt_bad_vertex_roi_roi_metric", None) is not None:
        cargs.extend([
            "-bad-vertex-roi",
            execution.input_file(params.get("opt_bad_vertex_roi_roi_metric", None))
        ])
    if params.get("opt_data_roi_roi_metric", None) is not None:
        cargs.extend([
            "-data-roi",
            execution.input_file(params.get("opt_data_roi_roi_metric", None))
        ])
    if params.get("opt_column_column", None) is not None:
        cargs.extend([
            "-column",
            params.get("opt_column_column", None)
        ])
    if params.get("opt_nearest", False):
        cargs.append("-nearest")
    if params.get("opt_linear", False):
        cargs.append("-linear")
    if params.get("opt_exponent_exponent", None) is not None:
        cargs.extend([
            "-exponent",
            str(params.get("opt_exponent_exponent", None))
        ])
    if params.get("opt_corrected_areas_area_metric", None) is not None:
        cargs.extend([
            "-corrected-areas",
            execution.input_file(params.get("opt_corrected_areas_area_metric", None))
        ])
    if params.get("opt_legacy_cutoff", False):
        cargs.append("-legacy-cutoff")
    return cargs


def metric_dilate_outputs(
    params: MetricDilateParameters,
    execution: Execution,
) -> MetricDilateOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetricDilateOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric_out", None)),
    )
    return ret


def metric_dilate_execute(
    params: MetricDilateParameters,
    runner: Runner | None = None,
) -> MetricDilateOutputs:
    """
    metric-dilate
    
    Dilate a metric file.
    
    For all metric vertices that are designated as bad, if they neighbor a
    non-bad vertex with data or are within the specified distance of such a
    vertex, replace the value with a distance-based weighted average of nearby
    non-bad vertices that have data, otherwise set the value to zero. No matter
    how small <distance> is, dilation will always use at least the immediate
    neighbor vertices. If -nearest is specified, it will use the value from the
    closest non-bad vertex with data within range instead of a weighted average.
    
    If -bad-vertex-roi is specified, all vertices with a positive ROI value are
    bad. If it is not specified, only vertices that have data, with a value of
    zero, are bad. If -data-roi is not specified, all vertices are assumed to
    have data.
    
    Note that the -corrected-areas option uses an approximate correction for the
    change in distances along a group average surface.
    
    To get the behavior of version 1.3.2 or earlier, use '-legacy-cutoff
    -exponent 2'.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricDilateOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_DILATE_METADATA)
    params = execution.params(params)
    cargs = metric_dilate_cargs(params, execution)
    ret = metric_dilate_outputs(params, execution)
    execution.run(cargs)
    return ret


def metric_dilate(
    metric: InputPathType,
    surface: InputPathType,
    distance: float,
    metric_out: str,
    opt_bad_vertex_roi_roi_metric: InputPathType | None = None,
    opt_data_roi_roi_metric: InputPathType | None = None,
    opt_column_column: str | None = None,
    opt_nearest: bool = False,
    opt_linear: bool = False,
    opt_exponent_exponent: float | None = None,
    opt_corrected_areas_area_metric: InputPathType | None = None,
    opt_legacy_cutoff: bool = False,
    runner: Runner | None = None,
) -> MetricDilateOutputs:
    """
    metric-dilate
    
    Dilate a metric file.
    
    For all metric vertices that are designated as bad, if they neighbor a
    non-bad vertex with data or are within the specified distance of such a
    vertex, replace the value with a distance-based weighted average of nearby
    non-bad vertices that have data, otherwise set the value to zero. No matter
    how small <distance> is, dilation will always use at least the immediate
    neighbor vertices. If -nearest is specified, it will use the value from the
    closest non-bad vertex with data within range instead of a weighted average.
    
    If -bad-vertex-roi is specified, all vertices with a positive ROI value are
    bad. If it is not specified, only vertices that have data, with a value of
    zero, are bad. If -data-roi is not specified, all vertices are assumed to
    have data.
    
    Note that the -corrected-areas option uses an approximate correction for the
    change in distances along a group average surface.
    
    To get the behavior of version 1.3.2 or earlier, use '-legacy-cutoff
    -exponent 2'.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        metric: the metric to dilate.
        surface: the surface to compute on.
        distance: distance in mm to dilate.
        metric_out: the output metric.
        opt_bad_vertex_roi_roi_metric: specify an roi of vertices to overwrite,\
            rather than vertices with value zero: metric file, positive values\
            denote vertices to have their values replaced.
        opt_data_roi_roi_metric: specify an roi of where there is data: metric\
            file, positive values denote vertices that have data.
        opt_column_column: select a single column to dilate: the column number\
            or name.
        opt_nearest: use the nearest good value instead of a weighted average.
        opt_linear: fill in values with linear interpolation along strongest\
            gradient.
        opt_exponent_exponent: use a different exponent in the weighting\
            function: exponent 'n' to use in (area / (distance ^ n)) as the\
            weighting function (default 6).
        opt_corrected_areas_area_metric: vertex areas to use instead of\
            computing them from the surface: the corrected vertex areas, as a\
            metric.
        opt_legacy_cutoff: use the v1.3.2 method of choosing how many vertices\
            to use when calulating the dilated value with weighted method.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricDilateOutputs`).
    """
    params = metric_dilate_params(
        metric=metric,
        surface=surface,
        distance=distance,
        metric_out=metric_out,
        opt_bad_vertex_roi_roi_metric=opt_bad_vertex_roi_roi_metric,
        opt_data_roi_roi_metric=opt_data_roi_roi_metric,
        opt_column_column=opt_column_column,
        opt_nearest=opt_nearest,
        opt_linear=opt_linear,
        opt_exponent_exponent=opt_exponent_exponent,
        opt_corrected_areas_area_metric=opt_corrected_areas_area_metric,
        opt_legacy_cutoff=opt_legacy_cutoff,
    )
    return metric_dilate_execute(params, runner)


__all__ = [
    "METRIC_DILATE_METADATA",
    "MetricDilateOutputs",
    "metric_dilate",
    "metric_dilate_execute",
    "metric_dilate_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_WEIGHTED_STATS_METADATA = Metadata(
    id="cc98b5f18ee1944d35c3bdc722bfbd19a6850252.boutiques",
    name="volume-weighted-stats",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


VolumeWeightedStatsWeightVolumeParameters = typing.TypedDict('VolumeWeightedStatsWeightVolumeParameters', {
    "@type": typing.NotRequired[typing.Literal["weight_volume"]],
    "weight_volume": InputPathType,
    "opt_match_maps": bool,
})
VolumeWeightedStatsWeightVolumeParametersTagged = typing.TypedDict('VolumeWeightedStatsWeightVolumeParametersTagged', {
    "@type": typing.Literal["weight_volume"],
    "weight_volume": InputPathType,
    "opt_match_maps": bool,
})


VolumeWeightedStatsRoiParameters = typing.TypedDict('VolumeWeightedStatsRoiParameters', {
    "@type": typing.NotRequired[typing.Literal["roi"]],
    "roi_volume": InputPathType,
    "opt_match_maps": bool,
})
VolumeWeightedStatsRoiParametersTagged = typing.TypedDict('VolumeWeightedStatsRoiParametersTagged', {
    "@type": typing.Literal["roi"],
    "roi_volume": InputPathType,
    "opt_match_maps": bool,
})


VolumeWeightedStatsStdevParameters = typing.TypedDict('VolumeWeightedStatsStdevParameters', {
    "@type": typing.NotRequired[typing.Literal["stdev"]],
    "opt_sample": bool,
})
VolumeWeightedStatsStdevParametersTagged = typing.TypedDict('VolumeWeightedStatsStdevParametersTagged', {
    "@type": typing.Literal["stdev"],
    "opt_sample": bool,
})


VolumeWeightedStatsParameters = typing.TypedDict('VolumeWeightedStatsParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/volume-weighted-stats"]],
    "volume_in": InputPathType,
    "weight_volume": typing.NotRequired[VolumeWeightedStatsWeightVolumeParameters | None],
    "opt_subvolume_subvolume": typing.NotRequired[str | None],
    "roi": typing.NotRequired[VolumeWeightedStatsRoiParameters | None],
    "opt_mean": bool,
    "stdev": typing.NotRequired[VolumeWeightedStatsStdevParameters | None],
    "opt_percentile_percent": typing.NotRequired[float | None],
    "opt_sum": bool,
    "opt_show_map_name": bool,
})
VolumeWeightedStatsParametersTagged = typing.TypedDict('VolumeWeightedStatsParametersTagged', {
    "@type": typing.Literal["workbench/volume-weighted-stats"],
    "volume_in": InputPathType,
    "weight_volume": typing.NotRequired[VolumeWeightedStatsWeightVolumeParameters | None],
    "opt_subvolume_subvolume": typing.NotRequired[str | None],
    "roi": typing.NotRequired[VolumeWeightedStatsRoiParameters | None],
    "opt_mean": bool,
    "stdev": typing.NotRequired[VolumeWeightedStatsStdevParameters | None],
    "opt_percentile_percent": typing.NotRequired[float | None],
    "opt_sum": bool,
    "opt_show_map_name": bool,
})


def volume_weighted_stats_weight_volume_params(
    weight_volume: InputPathType,
    opt_match_maps: bool = False,
) -> VolumeWeightedStatsWeightVolumeParametersTagged:
    """
    Build parameters.
    
    Args:
        weight_volume: volume file containing the weights.
        opt_match_maps: each subvolume of input uses the corresponding\
            subvolume from the weights file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "weight_volume",
        "weight_volume": weight_volume,
        "opt_match_maps": opt_match_maps,
    }
    return params


def volume_weighted_stats_weight_volume_cargs(
    params: VolumeWeightedStatsWeightVolumeParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-weight-volume")
    cargs.append(execution.input_file(params.get("weight_volume", None)))
    if params.get("opt_match_maps", False):
        cargs.append("-match-maps")
    return cargs


def volume_weighted_stats_roi_params(
    roi_volume: InputPathType,
    opt_match_maps: bool = False,
) -> VolumeWeightedStatsRoiParametersTagged:
    """
    Build parameters.
    
    Args:
        roi_volume: the roi, as a volume file.
        opt_match_maps: each subvolume of input uses the corresponding\
            subvolume from the roi file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "roi",
        "roi_volume": roi_volume,
        "opt_match_maps": opt_match_maps,
    }
    return params


def volume_weighted_stats_roi_cargs(
    params: VolumeWeightedStatsRoiParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-roi")
    cargs.append(execution.input_file(params.get("roi_volume", None)))
    if params.get("opt_match_maps", False):
        cargs.append("-match-maps")
    return cargs


def volume_weighted_stats_stdev_params(
    opt_sample: bool = False,
) -> VolumeWeightedStatsStdevParametersTagged:
    """
    Build parameters.
    
    Args:
        opt_sample: estimate population stdev from the sample.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "stdev",
        "opt_sample": opt_sample,
    }
    return params


def volume_weighted_stats_stdev_cargs(
    params: VolumeWeightedStatsStdevParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-stdev")
    if params.get("opt_sample", False):
        cargs.append("-sample")
    return cargs


class VolumeWeightedStatsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeWeightedStatsParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def volume_weighted_stats_params(
    volume_in: InputPathType,
    weight_volume: VolumeWeightedStatsWeightVolumeParameters | None = None,
    opt_subvolume_subvolume: str | None = None,
    roi: VolumeWeightedStatsRoiParameters | None = None,
    opt_mean: bool = False,
    stdev: VolumeWeightedStatsStdevParameters | None = None,
    opt_percentile_percent: float | None = None,
    opt_sum: bool = False,
    opt_show_map_name: bool = False,
) -> VolumeWeightedStatsParametersTagged:
    """
    Build parameters.
    
    Args:
        volume_in: the input volume.
        weight_volume: use weights from a volume file.
        opt_subvolume_subvolume: only display output for one subvolume: the\
            subvolume number or name.
        roi: only consider data inside an roi.
        opt_mean: compute weighted mean.
        stdev: compute weighted standard deviation.
        opt_percentile_percent: compute weighted percentile: the percentile to\
            find, must be between 0 and 100.
        opt_sum: compute weighted sum.
        opt_show_map_name: print map index and name before each output.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-weighted-stats",
        "volume_in": volume_in,
        "opt_mean": opt_mean,
        "opt_sum": opt_sum,
        "opt_show_map_name": opt_show_map_name,
    }
    if weight_volume is not None:
        params["weight_volume"] = weight_volume
    if opt_subvolume_subvolume is not None:
        params["opt_subvolume_subvolume"] = opt_subvolume_subvolume
    if roi is not None:
        params["roi"] = roi
    if stdev is not None:
        params["stdev"] = stdev
    if opt_percentile_percent is not None:
        params["opt_percentile_percent"] = opt_percentile_percent
    return params


def volume_weighted_stats_cargs(
    params: VolumeWeightedStatsParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-volume-weighted-stats")
    cargs.append(execution.input_file(params.get("volume_in", None)))
    if params.get("weight_volume", None) is not None:
        cargs.extend(volume_weighted_stats_weight_volume_cargs(params.get("weight_volume", None), execution))
    if params.get("opt_subvolume_subvolume", None) is not None:
        cargs.extend([
            "-subvolume",
            params.get("opt_subvolume_subvolume", None)
        ])
    if params.get("roi", None) is not None:
        cargs.extend(volume_weighted_stats_roi_cargs(params.get("roi", None), execution))
    if params.get("opt_mean", False):
        cargs.append("-mean")
    if params.get("stdev", None) is not None:
        cargs.extend(volume_weighted_stats_stdev_cargs(params.get("stdev", None), execution))
    if params.get("opt_percentile_percent", None) is not None:
        cargs.extend([
            "-percentile",
            str(params.get("opt_percentile_percent", None))
        ])
    if params.get("opt_sum", False):
        cargs.append("-sum")
    if params.get("opt_show_map_name", False):
        cargs.append("-show-map-name")
    return cargs


def volume_weighted_stats_outputs(
    params: VolumeWeightedStatsParameters,
    execution: Execution,
) -> VolumeWeightedStatsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeWeightedStatsOutputs(
        root=execution.output_file("."),
    )
    return ret


def volume_weighted_stats_execute(
    params: VolumeWeightedStatsParameters,
    runner: Runner | None = None,
) -> VolumeWeightedStatsOutputs:
    """
    volume-weighted-stats
    
    Weighted spatial statistics on a volume file.
    
    For each subvolume of the input, a line of text is printed, resulting from
    the specified operation. If -weight-volume is not specified, each voxel's
    volume is used. Use -subvolume to only give output for a single subvolume.
    If the -roi option is used without -match-maps, then each line will contain
    as many numbers as there are maps in the ROI file, separated by tab
    characters. Exactly one of -mean, -stdev, -percentile or -sum must be
    specified.
    
    Using -sum without -weight-volume is equivalent to integrating with respect
    to volume.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeWeightedStatsOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_WEIGHTED_STATS_METADATA)
    params = execution.params(params)
    cargs = volume_weighted_stats_cargs(params, execution)
    ret = volume_weighted_stats_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_weighted_stats(
    volume_in: InputPathType,
    weight_volume: VolumeWeightedStatsWeightVolumeParameters | None = None,
    opt_subvolume_subvolume: str | None = None,
    roi: VolumeWeightedStatsRoiParameters | None = None,
    opt_mean: bool = False,
    stdev: VolumeWeightedStatsStdevParameters | None = None,
    opt_percentile_percent: float | None = None,
    opt_sum: bool = False,
    opt_show_map_name: bool = False,
    runner: Runner | None = None,
) -> VolumeWeightedStatsOutputs:
    """
    volume-weighted-stats
    
    Weighted spatial statistics on a volume file.
    
    For each subvolume of the input, a line of text is printed, resulting from
    the specified operation. If -weight-volume is not specified, each voxel's
    volume is used. Use -subvolume to only give output for a single subvolume.
    If the -roi option is used without -match-maps, then each line will contain
    as many numbers as there are maps in the ROI file, separated by tab
    characters. Exactly one of -mean, -stdev, -percentile or -sum must be
    specified.
    
    Using -sum without -weight-volume is equivalent to integrating with respect
    to volume.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        volume_in: the input volume.
        weight_volume: use weights from a volume file.
        opt_subvolume_subvolume: only display output for one subvolume: the\
            subvolume number or name.
        roi: only consider data inside an roi.
        opt_mean: compute weighted mean.
        stdev: compute weighted standard deviation.
        opt_percentile_percent: compute weighted percentile: the percentile to\
            find, must be between 0 and 100.
        opt_sum: compute weighted sum.
        opt_show_map_name: print map index and name before each output.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeWeightedStatsOutputs`).
    """
    params = volume_weighted_stats_params(
        volume_in=volume_in,
        weight_volume=weight_volume,
        opt_subvolume_subvolume=opt_subvolume_subvolume,
        roi=roi,
        opt_mean=opt_mean,
        stdev=stdev,
        opt_percentile_percent=opt_percentile_percent,
        opt_sum=opt_sum,
        opt_show_map_name=opt_show_map_name,
    )
    return volume_weighted_stats_execute(params, runner)


__all__ = [
    "VOLUME_WEIGHTED_STATS_METADATA",
    "VolumeWeightedStatsOutputs",
    "volume_weighted_stats",
    "volume_weighted_stats_execute",
    "volume_weighted_stats_params",
    "volume_weighted_stats_roi_params",
    "volume_weighted_stats_stdev_params",
    "volume_weighted_stats_weight_volume_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_GEODESIC_ROIS_METADATA = Metadata(
    id="72311c67a08117facc775f0d49b6e99ecfb7beb1.boutiques",
    name="surface-geodesic-rois",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


SurfaceGeodesicRoisParameters = typing.TypedDict('SurfaceGeodesicRoisParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/surface-geodesic-rois"]],
    "surface": InputPathType,
    "limit": float,
    "vertex_list_file": str,
    "metric_out": str,
    "opt_gaussian_sigma": typing.NotRequired[float | None],
    "opt_overlap_logic_method": typing.NotRequired[str | None],
    "opt_names_name_list_file": typing.NotRequired[str | None],
    "opt_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
})
SurfaceGeodesicRoisParametersTagged = typing.TypedDict('SurfaceGeodesicRoisParametersTagged', {
    "@type": typing.Literal["workbench/surface-geodesic-rois"],
    "surface": InputPathType,
    "limit": float,
    "vertex_list_file": str,
    "metric_out": str,
    "opt_gaussian_sigma": typing.NotRequired[float | None],
    "opt_overlap_logic_method": typing.NotRequired[str | None],
    "opt_names_name_list_file": typing.NotRequired[str | None],
    "opt_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
})


class SurfaceGeodesicRoisOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceGeodesicRoisParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output metric"""


def surface_geodesic_rois_params(
    surface: InputPathType,
    limit: float,
    vertex_list_file: str,
    metric_out: str,
    opt_gaussian_sigma: float | None = None,
    opt_overlap_logic_method: str | None = None,
    opt_names_name_list_file: str | None = None,
    opt_corrected_areas_area_metric: InputPathType | None = None,
) -> SurfaceGeodesicRoisParametersTagged:
    """
    Build parameters.
    
    Args:
        surface: the surface to draw on.
        limit: geodesic distance limit from vertex, in mm.
        vertex_list_file: a text file containing the vertices to draw ROIs\
            around.
        metric_out: the output metric.
        opt_gaussian_sigma: generate a gaussian kernel instead of a flat ROI:\
            the sigma for the gaussian kernel, in mm.
        opt_overlap_logic_method: how to handle overlapping ROIs, default\
            ALLOW: the method of resolving overlaps.
        opt_names_name_list_file: name the columns from text file: a text file\
            containing column names, one per line.
        opt_corrected_areas_area_metric: vertex areas to use instead of\
            computing them from the surface: the corrected vertex areas, as a\
            metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-geodesic-rois",
        "surface": surface,
        "limit": limit,
        "vertex_list_file": vertex_list_file,
        "metric_out": metric_out,
    }
    if opt_gaussian_sigma is not None:
        params["opt_gaussian_sigma"] = opt_gaussian_sigma
    if opt_overlap_logic_method is not None:
        params["opt_overlap_logic_method"] = opt_overlap_logic_method
    if opt_names_name_list_file is not None:
        params["opt_names_name_list_file"] = opt_names_name_list_file
    if opt_corrected_areas_area_metric is not None:
        params["opt_corrected_areas_area_metric"] = opt_corrected_areas_area_metric
    return params


def surface_geodesic_rois_cargs(
    params: SurfaceGeodesicRoisParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-surface-geodesic-rois")
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(str(params.get("limit", None)))
    cargs.append(params.get("vertex_list_file", None))
    cargs.append(params.get("metric_out", None))
    if params.get("opt_gaussian_sigma", None) is not None:
        cargs.extend([
            "-gaussian",
            str(params.get("opt_gaussian_sigma", None))
        ])
    if params.get("opt_overlap_logic_method", None) is not None:
        cargs.extend([
            "-overlap-logic",
            params.get("opt_overlap_logic_method", None)
        ])
    if params.get("opt_names_name_list_file", None) is not None:
        cargs.extend([
            "-names",
            params.get("opt_names_name_list_file", None)
        ])
    if params.get("opt_corrected_areas_area_metric", None) is not None:
        cargs.extend([
            "-corrected-areas",
            execution.input_file(params.get("opt_corrected_areas_area_metric", None))
        ])
    return cargs


def surface_geodesic_rois_outputs(
    params: SurfaceGeodesicRoisParameters,
    execution: Execution,
) -> SurfaceGeodesicRoisOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceGeodesicRoisOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric_out", None)),
    )
    return ret


def surface_geodesic_rois_execute(
    params: SurfaceGeodesicRoisParameters,
    runner: Runner | None = None,
) -> SurfaceGeodesicRoisOutputs:
    """
    surface-geodesic-rois
    
    Draw geodesic limited rois at vertices.
    
    For each vertex in the list file, a column in the output metric is created,
    and an ROI around that vertex is drawn in that column. Each metric column
    will have zeros outside the geodesic distance spacified by <limit>, and by
    default will have a value of 1.0 inside it. If the -gaussian option is
    specified, the values inside the ROI will instead form a gaussian with the
    specified value of sigma, normalized so that the sum of the nonzero values
    in the metric column is 1.0. The <method> argument to -overlap-logic must be
    one of ALLOW, CLOSEST, or EXCLUDE. ALLOW is the default, and means that ROIs
    are treated independently and may overlap. CLOSEST means that ROIs may not
    overlap, and that no ROI contains vertices that are closer to a different
    seed vertex. EXCLUDE means that ROIs may not overlap, and that any vertex
    within range of more than one ROI does not belong to any ROI.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceGeodesicRoisOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_GEODESIC_ROIS_METADATA)
    params = execution.params(params)
    cargs = surface_geodesic_rois_cargs(params, execution)
    ret = surface_geodesic_rois_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_geodesic_rois(
    surface: InputPathType,
    limit: float,
    vertex_list_file: str,
    metric_out: str,
    opt_gaussian_sigma: float | None = None,
    opt_overlap_logic_method: str | None = None,
    opt_names_name_list_file: str | None = None,
    opt_corrected_areas_area_metric: InputPathType | None = None,
    runner: Runner | None = None,
) -> SurfaceGeodesicRoisOutputs:
    """
    surface-geodesic-rois
    
    Draw geodesic limited rois at vertices.
    
    For each vertex in the list file, a column in the output metric is created,
    and an ROI around that vertex is drawn in that column. Each metric column
    will have zeros outside the geodesic distance spacified by <limit>, and by
    default will have a value of 1.0 inside it. If the -gaussian option is
    specified, the values inside the ROI will instead form a gaussian with the
    specified value of sigma, normalized so that the sum of the nonzero values
    in the metric column is 1.0. The <method> argument to -overlap-logic must be
    one of ALLOW, CLOSEST, or EXCLUDE. ALLOW is the default, and means that ROIs
    are treated independently and may overlap. CLOSEST means that ROIs may not
    overlap, and that no ROI contains vertices that are closer to a different
    seed vertex. EXCLUDE means that ROIs may not overlap, and that any vertex
    within range of more than one ROI does not belong to any ROI.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        surface: the surface to draw on.
        limit: geodesic distance limit from vertex, in mm.
        vertex_list_file: a text file containing the vertices to draw ROIs\
            around.
        metric_out: the output metric.
        opt_gaussian_sigma: generate a gaussian kernel instead of a flat ROI:\
            the sigma for the gaussian kernel, in mm.
        opt_overlap_logic_method: how to handle overlapping ROIs, default\
            ALLOW: the method of resolving overlaps.
        opt_names_name_list_file: name the columns from text file: a text file\
            containing column names, one per line.
        opt_corrected_areas_area_metric: vertex areas to use instead of\
            computing them from the surface: the corrected vertex areas, as a\
            metric.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceGeodesicRoisOutputs`).
    """
    params = surface_geodesic_rois_params(
        surface=surface,
        limit=limit,
        vertex_list_file=vertex_list_file,
        metric_out=metric_out,
        opt_gaussian_sigma=opt_gaussian_sigma,
        opt_overlap_logic_method=opt_overlap_logic_method,
        opt_names_name_list_file=opt_names_name_list_file,
        opt_corrected_areas_area_metric=opt_corrected_areas_area_metric,
    )
    return surface_geodesic_rois_execute(params, runner)


__all__ = [
    "SURFACE_GEODESIC_ROIS_METADATA",
    "SurfaceGeodesicRoisOutputs",
    "surface_geodesic_rois",
    "surface_geodesic_rois_execute",
    "surface_geodesic_rois_params",
]
